<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version
 * @author Daniel Perez danielperez.arg@gmail.com
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/ActionManager.php');
require_once('core/UserPeer.php');
require_once('core/MonitorPeer.php');
require_once('propel/om/BaseObject.php');
require_once('propel/om/Persistent.php');
include_once('propel/util/Criteria.php');
require_once('notification/GFNotification.php');

class SiteNotification implements GFNotification {
	
	const MONITORED_USERS = "monitored_users";
	const ADMIN_USERS = "admin_users";
	const DEFAULT_USERSOPTION = "monitored_users";

	var $monitoring_users; // the users that are monitoring this
	var $message;
	var $subject;
	var $usersoption;

	/**
	 * Returns an array of user objects corresponding to the ones that are monitoring this package or project, or the ones who are 
	 * project admins
	 *
	 * @return array	Users
	 */
	function getUsers() {
		if ($this->usersoption == self::ADMIN_USERS){
			if (!$this->monitoring_users) {
				$this->monitoring_users = $this->getAdministrators();
			}
		} elseif ($this->usersoption == self::MONITORED_USERS) {
			if (!$this->monitoring_users) {
				$con = Propel::getConnection(UserPeer::DATABASE_NAME);
				//get frspackage monitors
				$sql = "SELECT * FROM " . UserPeer::TABLE_NAME  . " WHERE user_id IN (
					SELECT user_id FROM " . MonitorPeer::TABLE_NAME  . " WHERE
					(section='" . Monitor::SITE_SECTION . "') );";
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
				$this->monitoring_users = UserPeer::populateObjects($rs);
			}
		}
		return $this->monitoring_users;
	}

	/**
	 *	setNotificationMessage - construct a string of this message content.
	 *
	 *	@param	string	Message
	 *	@return string
	 */
 	function setNotificationMessage($Message) {
 		$url = GFUrl::getUrl();
		if ($Message !== '') {
			$Message .= "\n\n\n---------------------\n".Language::getSessText('SiteNotification.DisableMailLink');
			$Message .= "\n".$url->getFullURL($url->getMonitorUrl('site', '0', '/gf/my/monitors/', false), true);
		}
 		$this->message = $Message;
 	}

	/**
	 *	setNotificationSubject - construct a string of this subject content.
	 *
	 *	@param	string	Subject
	 *	@return string
	 */
	function setNotificationSubject($Subject) {
		$this->subject = $Subject;
	}	
	
	/**
	 *	getNotificationMessage - construct a string of this message content.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationMessage(User $User) {
		if ($this->message) {
			return $this->message;
		} else {
			return '';
		}
	}
	
	/**
	 *	getNotificationSubject - construct a string of this message subject.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationSubject(User $User) {
		if ($this->subject) {	
			return $this->subject;
		} else {
			return '';
		}
	}
	
	/**
	 *	getSection - retrieves the section name of the site 
	 *	@return string
	 */	
	function getSection() {
		return Monitor::SITE_SECTION;
	}
	
	/**
	 *	getRefId - retrieves 0
	 *	@return int
	 */	
	function getRefId() {
		return 0; // theres only one site, this is a dummy return
	}

	public function setUsersOption($v) {
		$this->usersoption = $v;
	}

	public function getUsersOption() {
		return $this->usersoption;
	}

	/**
	 *	Returns an array of User objects which are project administrators
	 *	@return array User
	 */	
	public function getAdministrators() {
		$sql = "SELECT DISTINCT u.*
FROM user_project up, user_project_role upr, \"user\" u, role_setting rs
WHERE up.user_project_id = upr.user_project_id
AND u.user_id = up.user_id
AND upr.role_id = rs.role_id
AND rs.section = 'projectadmin'
AND rs.ref_id=0
AND rs.value='1'";
		$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql,ResultSet::FETCHMODE_NUM);

		$projectAdmins = UserPeer::populateObjects($rs);

		return $projectAdmins;
	}

}
?>
