<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: ForumAdminFormRenderer.php 417 2006-02-10 19:33:48Z marcelo $
 * @author	Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('SystemUtils.php');
require_once('GFConf.php');
require_once('GFUrl.php');
require_once('session/Session.php');
require_once('html/Paginator.php');
require_once('html/HTMLWidget.php');
require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('plugins/forum/lib/ForumThread.php');
require_once('plugins/forum/lib/Forum.php');
require_once('lib/core/FilesystemPeer.php');
require_once('lib/core/Filesystem.php');
require_once('external/fckeditor/fckeditor.php');
require_once('securimage/securimage.php');
require_once('lib/html/association.php');
require_once('lib/html/TagsRenderer.php');

/**
 * The forums will be re-used in another section, so we wrap up all the pages in this
 * class so we can use them across the sections.
 */
class ForumFormRenderer {
	private $section = null;
	private $ref_id = null;
	private $wwwlibPath;
	private $template;
	private $language;
	private $errors = array();
	private $readonly = false;
	
	// Variables that will be stored on the querystring across forum pages
	private $kept_variables;
	private $url;
	
	/** Flat: one thread/message after another */
	const THREAD_STYLE_FLAT = 'flat';
	/** Threaded: Messges in thread displayed in tree-fashion (title + text) */
	const THREAD_STYLE_THREADED = 'threaded';

	/** Flat message: used when displayed in thread */
	const MESSAGE_STYLE_FLAT = 1;
	/** Reply message: used when replying a msesage*/
	const MESSAGE_STYLE_REPLY = 2;

	/** Sorting constants */
	const SORT_BY_THREAD_NAME = 'thread_name';
	const SORT_BY_CREATED_BY = 'created_by';
	const SORT_BY_MESSAGE_COUNT = 'message_count';
	const SORT_BY_RECENT_DATE = 'most_recent_date';
	const SORT_BY_PAGE_VIEWS = 'page_views';
	const SORT_ASC = 'asc';
	const SORT_DESC = 'desc';
	

	public function __construct($section, $ref_id, $template, $language, $kept_variables = array()) {
		$this->section = $section;
		$this->ref_id = $ref_id;
		$this->template = $template;
		$this->language = $language;
		$this->kept_variables = $kept_variables;
	
		$this->wwwlibPath = SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/forum/wwwlib/forms');
		$this->url = GFUrl::getUrl();

		$this->section_url = $this->section;
		if ($this->section == 'wikipage') {
			$this->section_url = 'wiki';
		}
	}
	
	/**
	 * Returns a table with all the forums
	 *
	 * @return string
	 */
	public function showForumBrowseForm() {
		$Lang = $this->language;
		$theme = $this->template;
		$url = GFUrl::getUrl();

		$columns = array(
			ForumPeer::FORUM_NAME => $Lang->getText('Forum.ForumNameName'),
			ForumPeer::THREAD_COUNT => $Lang->getText('Forum.ThreadCountName'),
			ForumPeer::MESSAGE_COUNT => $Lang->getText('Forum.MessageCountName'),
			ForumPeer::MOST_RECENT_DATE => $Lang->getText('Forum.MostRecentDateName')
		);
		
		// Retrieve the objects array
		$crit = new Criteria();
		$crit->add(ForumPeer::SECTION, $this->section);
		$crit->add(ForumPeer::REF_ID, $this->ref_id);
		$total = ForumPeer::doCount($crit);		
		$ret = '';
		$paginator = new Paginator('forum_browse', -1, $total, ForumPeer::FORUM_NAME, $columns);
		$paginator->addPageClauses($crit);
		$arr = ForumPeer::doSelectWithLatestMessage($crit);
		
		$theader = $paginator->getSortableTableHeaders($theme);
		$ret .= $theme->tableTop($theader);
		
		$has_forums = false;
		
		foreach ($arr as $item) {
			$forum = $item['forum'];
			$latest_message = $item['latest_message'];

			if ($forum->userCanRead()) {
				$has_forums = true;
				$trow = array();
				
				$trow[] = '<a href="'.$this->buildForumURL(array('action' => 'ForumBrowse', 'forum_id' => $forum->getForumId())).'"><img src="/images/forum_icon.gif" height="16" width="16" border="0" alt=""/>&nbsp;'.$forum->getForumName().'</a> <br />'.
						$forum->getDescription();
				$trow[] = $forum->getThreadCount();
				$trow[] = $forum->getMessageCount();

				if ($forum->getThreadCount() > 0) {
					$user = $latest_message->getUser();
					$trow[] = $Lang->getText('Forum.LatestPostDescription', array($forum->getMostRecentDate(), 
									'<a href="'.$url->userUrl($user).'">'.$user->getFullName().'</a>' ));
				} else {
					$trow[] = '-';
				}
				$ret .= $theme->tableRow($trow, array(), false);
			}
		}
		
		if (!$has_forums) {
			$ret .= $theme->tableFullRow($Lang->getText('Forum.NoneFound'));
		} 

		$project = GFUrl::getUrl()->getProject();
		$perm = $project->getPermission();
		if ($perm->check(Forum::SECTION, 0, Forum::ADMIN)) {
			$button_enabled = true;
		} else {
			$button_enabled = false;
		}

		$ret .= $theme->tableBottom(array(GFUrl::getURL()->projectUrl($project,'','forum','admin',array())=>$Lang->getText('Forum.ManageForums')),array($button_enabled));

		if (isset($paginator) && is_object($paginator)) {
			$ret .= '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
		}			

		return $ret;
	}
	
	/**
	 * Displays a table with all the threads.
	 */
	public function getThreads($forum) {
		$Lang = $this->language;
		$theme = $this->template;
		$session = Session::getSession();
		$url = GFUrl::getUrl();
		
		if (!$forum instanceof Forum) {
			$forum_id = $forum;
			// first make sure the forum is correct
			$forum = ForumPeer::retrieveByPk($forum_id);
			if (!is_object($forum) || ($forum->getSection() != $this->section) || ($forum->getRefId() != $this->ref_id)) {
				throw new PermissionDeniedException('Invalid forum ID: '.$forum_id.' for section "'.$this->section.'" and reference ID '.$this->ref_id);
			}
		} else {
			$forum_id = $forum->getForumId();
		}
		
		$svdt = explode('|',$forum->getSavedDate(Session::getSession()->getUser()));
		$saved_date = (isset($svdt[1]) ? $svdt[1] : $svdt[0]);
		
		// Retrieve the objects array, paginated
		

		// Get all the threads, paginated	
		$c = new Criteria();
		$c->add(ForumThreadPeer::FORUM_ID, $forum->getForumId());
		$c->add(ForumThreadPeer::MESSAGE_COUNT, 0, Criteria::GREATER_THAN);
		$c->addJoin(UserPeer::USER_ID, ForumThreadPeer::CREATED_BY);
		$c->addDescendingOrderByColumn(ForumThreadPeer::IS_STICKY);		// Sticky threads must show on top
		
		$columns = array(
			1							=> $Lang->getText('ForumThread.Locked'), 
			2							=> $Lang->getText('ForumThread.Sticky'),
			self::SORT_BY_THREAD_NAME => $Lang->getText('ForumThread.ThreadNameName'),
			self::SORT_BY_CREATED_BY => $Lang->getText('ForumThread.CreatedByName'),
			self::SORT_BY_MESSAGE_COUNT => $Lang->getText('ForumThread.MessageCountName'),
			self::SORT_BY_PAGE_VIEWS	=> $Lang->getText('ForumThread.PageViewsName'),
			self::SORT_BY_RECENT_DATE => $Lang->getText('ForumThread.MostRecentDateName')
			);
			
		$total = ForumThreadPeer::doCount($c);
		
		$this->paginator = new Paginator('forum_thread', -1, $total, self::SORT_BY_RECENT_DATE, $columns);
		$limit = $this->getThreadLimit();
		//$output = $this->showLimitOptions();
		$output = '';
		$this->paginator->addPageClauses($c);
		
		$arr = ForumThreadPeer::doSelect($c);

		// Show the table header with the sorting options
		$disabled_vars = array('forum_change_view');	// Variables we should NOT pass (mostly related with the paginator)
		
		$theader = $this->paginator->getSortableTableHeaders($theme, $disabled_vars);
	
		$output .= $theme->tableTop($theader);
	
		if (count($arr) == 0) {
			$output .= $theme->tableFullRow('<span class="nonefound">'.$Lang->getText('ForumThread.NoneFound').'</span>');
		} else {
			foreach ($arr as $item) {
				$trow = array();
				$atts = array();
				
				if (!$forum->userIsAdmin()) {
					$trow[] = $item->getIsLocked() ? '<img src="/images/locked.png" alt="locked" />' : '&nbsp;';
					$atts[] = 'width="5%" align="center"';
					$trow[] = $item->getIsSticky() ? '<img src="/images/sticky.gif" alt="sticky" />' : '&nbsp;';
					$atts[] = 'width="5%" align="center"';
				} else {
					$trow[] = $item->getIsLocked() ? '<a href="'.$this->buildForumURL(array('_forum_action' => 'LockThread', 'lock' => 0, 'thread_id' => $item->getForumThreadId(), 'return_action' => 'ForumBrowse')).'" alt="Unlock"><img src="/images/locked.png" alt="locked" border="0" /></a>' : '&nbsp;';
					$atts[] = 'width="5%" align="center"';
					$trow[] = $item->getIsSticky() ? '<a href="'.$this->buildForumURL(array('_forum_action' => 'StickyThread', 'sticky' => 0, 'thread_id' => $item->getForumThreadId(), 'return_action' => 'ForumBrowse')).'" alt="Unmark as sticky"><img src="/images/sticky.gif" alt="locked" border="0" /></a>' : '&nbsp;';
					$atts[] = 'width="5%" align="center"';
				}
				
				$unread = ($saved_date < $item->getMostRecentDate(null));
				if ($unread) {
					$text='<strong><a href="'.$this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 'thread_id'=>$item->getForumThreadId())).'"> '.$item->getThreadName().'</a></strong>';
				} else {
					$text='<a href="'.$this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 'thread_id'=>$item->getForumThreadId())).'"> '.$item->getThreadName().'</a>';
				}
				
				$trow[] = $text;
				$atts[] = 'width="%40"';
				
				$user = $item->getUser();
				if ($user->isUndefinedUser()) {
					// undefined user?
					$trow[] = $item->getUndefinedUserName();
				} else {
					$trow[] = '<a href="'.$url->userUrl($user).'">'.$user->getFullName().'</a>';	// retrieves the created_by field
				}
				$atts[] = 'width="20%"';
				$trow[] = $item->getMessageCount();
				$atts[] = 'width="10%"';
				$trow[] = $item->getPageViews();
				$atts[] = 'width="10%"';
				$trow[] = $item->getMostRecentDate();
				$atts[] = 'width="10%"';
		
				$output .= $theme->tableRow($trow, $atts, false);
			}
		}
		
		//$output .= $theme->listPaginatorRow($this->paginator);		
		if ($forum->userCanPost($session->getUser()) && !$this->readonly) {
			$button_enabled = true;
		} else {
			$button_enabled = false;
		}

		$output .= $theme->tableBottom($this->getForumButtons($forum),array($button_enabled));	
			
		if (isset($this->paginator) && is_object($this->paginator)) {
			$output .= '<div class="paginator">' . $this->paginator->getPaginator() . '</div><br />';
		}
		
		// Set the saved date as 10 minutes ago
		$user = Session::getSession()->getUser();
		if ($user) {
			//if last visit greater than 30 minutes ago, set the last visit date to now
			$svdt=explode('|',$forum->getSavedDate($user));
			if ($svdt[0] < (time()-(30*60))) {
				$forum->setSavedDate($user, time().'|'.$svdt[0]);
			}
		}

		return $output;
	}

	/**
	 * Returns true if the form was processed and errors were found
	 * @return bool
	 */
	public function hasErrors() {
		return (count($this->errors) > 0);
	}
	
	/**
	 * Returns an array with the errors found after processing the POST data
	 * @return array
	 */
	public function getErrors() {
		return $this->errors;
	}

	/**
	 * Returns a form for creating a new thread
	 *
	 * @return	strings
	 */
	public function newThreadForm($Forum, $redirect_to, $force_form=false) {
		if ($this->readonly) {
			throw new PermissionDeniedException();
		}

		if ($Forum instanceof Forum) {
			$forum_id = $Forum->getForumId();
		} else {
			$forum_id = $Forum;
			$Forum = ForumPeer::retrieveByPk($forum_id);
		}
		
		
		$theme = $this->template;
		$Lang = $this->language;
		$session = Session::getSession();

		if (!is_object($Forum)) {
			throw new PermissionDeniedException();
		} else if ($Forum->getSection() != $this->section || $Forum->getRefId() != $this->ref_id) {
			throw new PermissionDeniedException($Lang->getText('ForumThread.InvalidRefId'));
		} else if (!$session->isLoggedIn() || !$Forum->userCanPost($session->getUser())) {
			throw new PermissionDeniedException();
		}

		// See if we've submitted the form. In that case, process the data.
		if (isset($_POST['forum_thread_add_submit']) && !$force_form) {
			return $this->postNewThread($Forum, $redirect_to);
		}
		
		$thread_name = ((isset($_POST['thread_name'])) ? htmlspecialchars($_POST['thread_name']) : '' );
		$message = ((isset($_POST['message'])) ? htmlspecialchars($_POST['message']) : '' );

		$formFile = SystemUtils::buildFilePath($this->wwwlibPath, 'ForumThreadAdd-tpl.php');
		ob_start();	
		include($formFile);
		$form = ob_get_clean();
		
		return $form;
	}
	
	private function postNewThread(Forum $forum, $redirect_to) {
		if ($this->readonly) {
			throw new PermissionDeniedException();
		}

		//if captcha is enabled, checks captcha code entered
		$bol_captcha_accepted = true;
		if (GFConf::get('forum.useCaptcha') == true) { 
			//Get form captcha
			$captcha_code = isset($_POST['captcha_code']) ? $_POST['captcha_code'] : 0;
			$securimage = new Securimage();
			//Verify captcha
			if ($securimage->check($captcha_code) == false) {
				$bol_captcha_accepted = false;
				$this->errors = array(Language::getSessText('Captcha.DontMatch'));
			}
		}

		if ($bol_captcha_accepted == true) {
			try {
				$thread_name = Validator::sanitizeString($_POST['thread_name']);
				$message = Validator::sanitizeHtml($_POST['message']);
				$newmsg=$forum->startThread($thread_name, $message);
			} catch (InvalidDataException $ex) {
				$this->errors = $ex->getErrors();
			}
		}

		if ($this->hasErrors()) {
			/*$errors = $this->getErrors();
			$feedback = '* '.implode('<br />* ', $errors);
			GFUrl::redirect($redirect_to, array('feedback' => $feedback));*/
			// show the form again
			return $this->newThreadForm($forum, $redirect_to, true);
		} else {
			if ($newmsg->getIsApproved()) {
				GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('ForumThread.AddFeedback')));
			} else {
				GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('Forum.MessageQueuedForApproval')));
			}
		}
	}
	
	/**
	 * Checks that the tread is valid inside this forum, and returns the thread object.
	 *
	 * @param	int $thread_id
	 * @return	ForumThread
	 */
	private function validateThread($thread_id) {
		$c = new Criteria();
		$c->add(ForumThreadPeer::FORUM_THREAD_ID, $thread_id);
		$thread = ForumThreadPeer::doSelectJoinForum($c);
		
		if (count($thread) == 0) {
			throw new PermissionDeniedException($Lang->getText('ForumThread.InvalidThreadID'));
		}
		
		$thread = $thread[0];
		
		$ref_id = $thread->getForum()->getRefId();
		$section = $thread->getForum()->getSection();
		
		if ($ref_id != $this->ref_id || $section != $this->section) {
			throw new PermissionDeniedException($this->language->getText('ForumThread.InvalidRefId'));
		}
		
		return $thread;
	}
	
	/**
	 * Displays the messages inside a thread.
	 */
	public function getThreadMessages($thread) {
		$theme = $this->template;
		$Lang = $this->language;
		
		if (!($thread instanceof ForumThread)) {
			// $thread is the thread_id instead of the thread object
			$thread = $this->validateThread($thread);
		}
		
		$output = $this->showViewsOptions();
		
 		$style = $this->getThreadStyle();
 		switch ($style) {
			default: 
			case ForumFormRenderer::THREAD_STYLE_FLAT: {
				$output .= $this->showFlatMessages($thread);
				break;
			}
			case ForumFormRenderer::THREAD_STYLE_THREADED: {
				$output .= $this->showThreadedMessages($thread);
				break;
			}
		}
		
		// Increase the views count
		$thread->setPageViews($thread->getPageViews()+1);
		$thread->save();
		
		return $output;
	}
	
	private function showFlatMessages(ForumThread $thread) {
		$session = Session::getSession();
		$Lang = $this->language;
		$forum = $thread->getForum();
		$forum_id = $forum->getForumId();
		$url = GFUrl::getUrl();
		
		//$saved_date = $forum->getSavedDate($session->getUser());
		$output = '';

		$messages = $thread->getThreadMessagesFlat();
		foreach ($messages as $message) {
			$output .=  $this->getMessage($message);
			$output .= '<br />'."\n";
		}
		
		//$output .= '<div align="center"><a href="'.$url->currentUrl(array('_forum_action' => 'ForumThreadBrowse')).'">'.$Lang->getText('ForumMessage.BackToThreads').'</a>';
		$output .='<div align="right">' . HTMLWidget::button("button",$Lang->getText('ForumMessage.BackToThreads'),"cancel","","window.location='".$this->buildForumURL(array('action' => 'ForumBrowse', 'forum_id' => $forum_id))."';") . '</div>';
		
		return $output;
	}

	private function getMessage(ForumMessage $message, $style = ForumFormRenderer::MESSAGE_STYLE_FLAT) {
		$Lang = $this->language;
		$url = GFUrl::getUrl();
		$user = $message->getUser();
		$session = Session::getSession();
		$theme = $session->getTheme();
		$forum = $message->getForumThread()->getForum();
		$thread = $message->getForumThread();
		
		$return = '';
		
		if ($user->isUndefinedUser()) {
			$posted_by = $message->getMailSender();
		} else {
			$posted_by = '<a href="'.$url->userUrl($user).'">'.$user->getFullName().'</a>';
		}		

		$message_header = 'Posted By: '.$posted_by.'<br />
							Join Date: '.$user->getCreateDate('Y-m-d').'<br />
							Location: '.$user->getCountryCode()->getCountryName().'<br />
		Message ID: '.$message->getForumMessageId();

		$footer=array();

		// All buttons disabled by default
		$buttons_enabled = array();
		
//		if (!$thread->getIsLocked() || $forum->userIsAdmin()) {
			if (!$this->readonly && $session->isLoggedIn() && $forum->userCanPost($session->getUser()) && !$thread->getIsLocked()) {
				$footer[$this->buildForumURL(array('_forum_action' => 'MessageReply', 'message_id'=>$message->getForumMessageId(), 'quote'=>1, 'thread_id'=>$message->getForumThread()->getForumThreadId()))] = $Lang->getText('ForumMessage.Quote');
				$buttons_enabled[] = true;
				$footer[$this->buildForumURL(array('_forum_action' => 'MessageReply', 'message_id'=>$message->getForumMessageId(), 'thread_id'=>$message->getForumThread()->getForumThreadId()))] = $Lang->getText('ForumMessage.Reply');
				$buttons_enabled[] = true;
			} else {
				$footer['disabled_1'] = $Lang->getText('ForumMessage.Quote');
				$buttons_enabled[] = false;
				$footer['disabled_2'] = $Lang->getText('ForumMessage.Reply');				
				$buttons_enabled[] = false;
			}
			
			if (!$this->readonly && $message->userCanUpdate() && !$thread->getIsLocked()) {
				$footer[$this->buildForumURL(array('_forum_action' => 'MessageEdit', 'message_id' => $message->getForumMessageId()), array('thread_id'))] = $Lang->getText('ForumMessage.Edit');
				$buttons_enabled[] = true;
			} else {
				$footer['disabled_3'] = $Lang->getText('ForumMessage.Edit');
				$buttons_enabled[] = false;
			}
			
			if (!$this->readonly && $message->userCanDelete() && !$thread->getIsLocked()) {
				$footer[$this->buildForumURL(array('_forum_action' => 'MessageDelete', 'message_id' => $message->getForumMessageId()), array('thread_id'))] = $Lang->getText('ForumMessage.Delete');
				$buttons_enabled[] = true;
			} else {
				$footer['disabled_4'] = $Lang->getText('ForumMessage.Delete');
				$buttons_enabled[] = false;
			}
	
//		}
		
		// Sticky
		if ($message->getParentForumMessageId() == 0 && $forum->userIsAdmin() && !$thread->getIsLocked()) {
			if ($thread->getIsSticky()) {
				$footer[$this->buildForumURL(array('_forum_action' => 'StickyThread', 'sticky' => 0, 'thread_id' => $message->getForumThread()->getForumThreadId()))] = $Lang->getText('ForumThread.UnsetSticky');
			} else {
				$footer[$this->buildForumURL(array('_forum_action' => 'StickyThread', 'sticky' => 1, 'thread_id' => $message->getForumThread()->getForumThreadId()))] = $Lang->getText('ForumThread.SetSticky');
			}
			$buttons_enabled[] = true;
		}

		// Thread locking
		if ($message->getParentForumMessageId() == 0 && $forum->userIsAdmin()) {
			if ($thread->getIsLocked()) {
				$footer[$this->buildForumURL(array('_forum_action' => 'LockThread', 'lock' => 0, 'thread_id' => $message->getForumThread()->getForumThreadId()))] = $Lang->getText('Forum.UnlockThread');
			} else {
				$footer[$this->buildForumURL(array('_forum_action' => 'LockThread', 'lock' => 1, 'thread_id' => $message->getForumThread()->getForumThreadId()))] = $Lang->getText('Forum.LockThread');
			}
			$buttons_enabled[] = true;
		}

		$return .= $theme->tableTop(array($message->getPostDate('Y-m-d H:i:s'),$message->getSubject()),array(),'width="100%"');
		$crit = new Criteria();
		$crit->add(FilesystemPeer::SECTION,ForumMessage::SECTION);
		$crit->add(FilesystemPeer::REF_ID,$message->getForumMessageId());
		$fs=FilesystemPeer::doSelect($crit);
		$msgtxt=$message->getBody();
		if (count($fs) > 0) {
			$msgtxt.='<br /><br />Attachments:';
			foreach ($fs as $f) {
				$url = GFUrl::getURL()->downloadURL(ForumMessage::SECTION,$message->getForumMessageId(),$f->getPrimaryKey(),$f->getFileNameSafe());
				$msgtxt .=  '<br /><img src="/images/docman_doc.gif" width="16" height="16" alt="" /><a href="'.$url.'">'.$f->getFileName().'</a>';
			}
		}
		$return .= $theme->tableRow(array($message_header,$msgtxt),array('nowrap="nowrap"','width="80%"'),true);
		$return .= $theme->tableBottom($footer, $buttons_enabled);
			
		return $return;
	}

	/**
	 * Checks that the message is valid inside this forum, and returns the ForumMessage object
	 * @param	int $message_id
	 * @return	ForumMessage
	 */
	private function validateMessage($message_id) {
		$c = new Criteria();
		$c->add(ForumMessagePeer::FORUM_MESSAGE_ID, $message_id);
		$messages = ForumMessagePeer::doSelectJoinForumThread($message_id);
		
		if (count($messages) == 0) {
			throw new PermissionDeniedException($Lang->getText('Forum.InvalidForumMessageId'));
		}
		
		$message = $messages[0];
		$forum = $message->getForumThread()->getForum();
		
		$ref_id = $forum->getRefId();
		$section = $forum->getSection();
		
		if ($ref_id != $this->ref_id || $section != $this->section) {
			throw new PermissionDeniedException($Lang->getText('ForumMessage.InvalidRefId'));
		}
		
		return $message;
	}

	public function messageReplyForm($message, $redirect_to, $force_form=false) {
		$lang = $this->language;
		$url = GFUrl::getUrl();
		$session = Session::getSession();

		if (!($message instanceof ForumMessage)) {
			$message = $this->validateMessage($message);
		}
		
		$forum = $message->getForumThread()->getForum();
		$forum_id = $forum->getForumId();
		$thread_id = $message->getForumThread()->getForumThreadId();
		
		// checks that the user can post messages in the forum
		if (!is_object($forum)) {
			throw new PermissionDeniedException();
		} else if ($forum->getSection() != $this->section || $forum->getRefId() != $this->ref_id) {
			throw new PermissionDeniedException($lang->getText('ForumThread.InvalidRefId'));
		}
				
		// See if we've submitted the form. In that case, process the data.
		if (isset($_POST['forum_post_reply']) && !$force_form) {
			return $this->postMessageReply($message, $redirect_to);
		}

		$return = '';
		
		$return .= $this->getMessage($message, ForumFormRenderer::MESSAGE_STYLE_REPLY);
		$return .= '<br />';
		$return .= $this->getMessageThread($message);
		
		// If this is readonly, don't allow posting
		if ($this->readonly || !$session->isLoggedIn() || !$forum->userCanPost($session->getUser())) {
			return $return;
		}
		
		if (empty($_POST['subject'])) {
			$prefix = $lang->getText('ForumMessage.ReplyPrefix');
			$subject = $message->getSubject();
			if (preg_match('/^[[:blank:]]*'.strtolower($prefix).'/', strtolower($subject))) {
				$reply_subject = $subject;
			} else {
				$reply_subject = $prefix.' '.$subject;
			}
		} else {
			$reply_subject = htmlspecialchars($_POST['subject']);
		}
		
		if (!empty($_POST['body'])) {
			$reply_body = htmlspecialchars($_POST['body']);
		} else {
//if quoting the prior message, include it here with a break
			if (isset($_GET['quote']) && $_GET['quote']) {
				$reply_body = $message->getBody().'
---

';
			} else {
//else it's a blank message
				$reply_body = '';
			}
		}
		
		$oFCKeditor = new FCKeditor('body') ;
		$oFCKeditor->BasePath = GFUrl::getUrl()->buildURL('/fckeditor/');
		$oFCKeditor->Value = htmlspecialchars_decode($reply_body);
		$oFCKeditor->Width = "700";
		$oFCKeditor->Height = "500";
		$oFCKeditor->ToolbarSet = "GForge";

		$return .= '<br />
			<form name="forumpost" id="forumpost" method="post" enctype="multipart/form-data">
			<p>
				<table border="0" align="center">
				<tr><td align="center">
				<h3>'.$lang->getText('ForumMessage.PostFollowup').'</h3>
				</td></tr>
				<tr><td>
					<strong>'.$lang->getText('ForumMessage.SubjectName').':</strong><br />
					<input type="text" name="subject" value="'.$reply_subject.'" size="60" maxlength="90" />
				</td></tr>
				<tr><td>
					<strong>'.$lang->getText('ForumMessage.BodyName').':</strong><br />';
//					<textarea name="body" rows="15" cols="60">'.$reply_body.'</textarea>
		$return .= $oFCKeditor->CreateHtml();
		$return .= '
				</td></tr>
				<tr><td>
					<strong>'. $lang->getText('Filesystem.AddNew') .'</strong><br />
					<input name="addfile[]" size="30" type="file"><br />
					<input name="addfile[]" size="30" type="file"><br />
					<input name="addfile[]" size="30" type="file"><br />
					<input name="addfile[]" size="30" type="file"><br />
					<input name="addfile[]" size="30" type="file">
				</td></tr>';

		if (GFConf::get('forum.useCaptcha') == true) {
			$return .= '
				<tr align="left">
					<td><strong>' . $lang->getText('Captcha.Captcha') . '</strong><span class="required">*</span><br />
						<img src="' . GFUrl::getUrl()->baseUrl() . '>?action=secureimage&sid=' .  md5(uniqid(time())) . '" id="image" align="absmiddle" />
						<a href="' . GFUrl::getUrl()->baseUrl() . '?action=secureimageaudio&sid=' . md5(uniqid(time())) . '" style="font-size: 11px">(Click here for audio version)</a><br /><br />
						<a href="#" onclick="document.getElementById(\'image\').src = \'' . GFUrl::getUrl()->baseUrl() . '?action=secureimage&sid=\' + Math.random(); return false">Request new image</a>
					</td>
				</tr>
				<tr align="left">
					<td><strong>' . $lang->getText('Captcha.EnterCaptcha') . '</strong><span class="required">*</span><br />
						<input type="text" name="captcha_code" size="10" maxlength="6" />
					</td>
				</tr>';
		}

		$return .= '<tr><td align="center">
					'.HTMLWidget::button("submit",$lang->getText('ForumMessage.PostReply'),"forum_post_reply","","").'
					'.HTMLWidget::button("button",$lang->getText('Common.Back'),"back","","window.location='".$this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 'action' => 'ForumBrowse', 'forum_id' => $forum_id, 'thread_id' => $thread_id))."';").'
				</td></tr>
				</table>
			</p>
			</form>';
			
		return $return;
	}
	
	private function postMessageReply(ForumMessage $message, $redirect_to) {
		$session = Session::getSession();
		
		if ($this->readonly || !$session->isLoggedIn()) {
			throw new PermissionDeniedException();
		}
		
		try {
			$con = Propel::getConnection();
		//	$con->begin();

			//if captcha is enabled, checks captcha code entered
			if (GFConf::get('forum.useCaptcha') == true) { 
				//Get form captcha
				$captcha_code = isset($_POST['captcha_code']) ? $_POST['captcha_code'] : 0;
				$securimage = new Securimage();
				//Verify captcha
				if ($securimage->check($captcha_code) == false) {
					$err = array(Language::getSessText('Captcha.DontMatch'));
					throw new InvalidDataException($err);
				}
			}

			$thread = $message->getForumThread();

			$subject = Validator::sanitizeString($_POST['subject']);
			$body = Validator::sanitizeHtml($_POST['body']);
			
			$new_message = new ForumMessage();
			$new_message->setUser($session->getUser());
			$new_message->setSubject($subject);
			$new_message->setBody($body);
			$new_message->setParentForumMessageId($message->getForumMessageId());
			
			$thread->addForumMessage($new_message);
			$thread->save();
			//
			//      handle the files
			//
			for ($i=0; $i<5; $i++) {
				$error=$_FILES['addfile']['error'][$i];
				$filename=$_FILES['addfile']['name'][$i];

				if ($error > 0 && $error != 4) {
					$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
					throw new InvalidDataException($fileerrors);
					continue;
				}

				$tmp_name=$_FILES['addfile']['tmp_name'][$i];
				if (!is_uploaded_file($tmp_name)) {
					continue;
				}
				$size=$_FILES['addfile']['size'][$i];
				$type=$_FILES['addfile']['type'][$i];

				try {
					$fs = new Filesystem();
					$fs->setSection(ForumMessage::SECTION);
					$fs->setRefId($new_message->getForumMessageId());

					$fs->setFileName( $filename );
					//$fs->setFileNameSafe();
					$fs->setFileType( $type );
					$fs->setFileSize( $size );
					$fs->setPostedBy( $session->getUser()->getUserId() );
					$fs->save();
					$fs->setFileDataFromPath( $tmp_name );
				} catch (Exception $e) {
					// rollback if we couldn´t create a file (write permissions)
					$con->rollback();
					$err = array($e->getMessage());
					throw new InvalidDataException($err);
				}
			}

		//	$con->commit();
		} catch (InvalidDataException $ex) {
			$this->errors = $ex->getErrors();
		}
		
		if ($this->hasErrors()) {
			/*$errors = $this->getErrors();
			$feedback = '* '.implode('<br />* ', $errors);
			GFUrl::redirect($redirect_to, array('feedback' => $feedback));*/
			return $this->messageReplyForm($message, $redirect_to, true);
		} else {
			if ($new_message->getIsApproved()) {
				GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('ForumMessage.AddFeedback')));
			} else {
				GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('Forum.MessageQueuedForApproval')));
			}
		}
	}
	
	private function showViewsOptions() {
		$lang = $this->language;
		$output = '';
		
		$output .= '<form method="get">';
		
		// we must pass all the _GET variables again
		foreach ($_GET as $var => $val) {
			if ($var == 'forum_thread_style' || $var == 'forum_thread_limit' || $var == 'feedback') continue;
			$output .= '<input type="hidden" name="'.$var.'" value="'.htmlspecialchars($val).'">'."\n";
		}
		
		$styles = array(
			self::THREAD_STYLE_FLAT => $lang->getText('ForumDisplayOptions.Flat'),
//			self::THREAD_STYLE_NESTED => $lang->getText('ForumDisplayOptions.Nested'),
			self::THREAD_STYLE_THREADED => $lang->getText('ForumDisplayOptions.Threaded')
		);
		
		$default_style = $this->getThreadStyle();
		$output .= '<select name="forum_thread_style">'."\n";
		foreach ($styles as $style => $name) {
			if ($style == $default_style) {
				$selected = 'selected="selected"';
			} else {
				$selected = '';
			}
			$output .= '<option value="'.$style.'" '.$selected.'>'.$name.'</option>'."\n";
		}
		$output .= '</select>'."\n";
		
		$output .= '&nbsp;&nbsp;';
		$output .=  HTMLWidget::button("submit",$lang->getText('ForumDisplayOptions.ChangeView'),"forum_change_view","","") ;
		
		$output .= '</form>';
		$output .= '<br />'."\n";
		
		return $output;
	}
	
	public function showLimitOptions() {
		return $this->paginator->showLimitOptions(array('forum_thread_style'));
		/*$lang = $this->language;
		$output = '';
		$output .= '<form method="get">';
		// we must pass all the _GET variables again
		foreach ($_GET as $var => $val) {
			if ($var == 'forum_thread_style' || $var == 'forum_thread_limit') continue;
			$output .= '<input type="hidden" name="'.$var.'" value="'.htmlspecialchars($val).'">'."\n";
		}


		$default_limit = $this->getThreadLimit();		
		$limits = array(25, 50, 75, 100);
		$output .= '<select name="forum_thread_limit">'."\n";
		foreach ($limits as $limit) {
			if ($limit == $default_limit) {
				$selected = 'selected="selected"';
			} else {
				$selected = '';
			}
			$output .= '<option value="'.$limit.'" '.$selected.'>'.$lang->getText('ForumDisplayOptions.Limit', array($limit)).'</option>'."\n";
		}
		$output .= '</select>'."\n";
		
		$output .= '&nbsp;&nbsp;';
		$output .= ' HTMLWidget::button("submit",$lang->getText('ForumDisplayOptions.Change'),"forum_change_view","","") ';
		
		$output .= '</form>';
		$output .= '<br />'."\n";
		return $output;*/
	}
	
	private function getThreadStyle() {
		$user = Session::getSession()->getUser();

		if (isset($_REQUEST['forum_thread_style'])) {
			$style = $_REQUEST['forum_thread_style'];
			if (!is_null($user) && ($user->getPreference(Forum::PREFERENCE_THREAD_STYLE) != $style) ) {
				// Saves the new style as a preference for the user
				$user->setPreference(Forum::PREFERENCE_THREAD_STYLE, $style);
			} else {
				$_SESSION['forum_thread_style'] = $style;
			}
			return $style;
		} elseif (!is_null($user)) {
			$style = $user->getPreference(Forum::PREFERENCE_THREAD_STYLE);
			if (!$style) {
				return self::THREAD_STYLE_FLAT;
			} else {
				return $style;
			}
		} else if (isset($_SESSION['forum_thread_style'])) {
			return $_SESSION['forum_thread_style'];
		} else {
			return self::THREAD_STYLE_FLAT;
		}
	}

	private function getThreadLimit() {
		/*$user = Session::getSession()->getUser();
		$default_limit = 50;

		if (isset($_REQUEST['forum_thread_limit'])) {
			$limit = $_REQUEST['forum_thread_limit'];
			if (!is_null($user) && ($user->getPreference(Forum::PREFERENCE_THREAD_LIMIT) != $limit) ) {
				// Saves the new style as a preference for the user
				$user->setPreference(Forum::PREFERENCE_THREAD_LIMIT, $limit);
			}
			return $limit;
		} elseif (!is_null($user)) {
			$limit = $user->getPreference(Forum::PREFERENCE_THREAD_LIMIT);
			if (!$limit) {
				return $default_limit;
			} else {
				return $limit;
			}
		} else {
			return $default_limit;
		}*/
		return $this->paginator->getLimit();
	}
	
	private function getSortingOptions() {
		/*$user = Session::getSession()->getUser();
		$default_sort_by = self::SORT_BY_RECENT_DATE;
		$default_order = self::SORT_DESC;
		
		if (isset($_REQUEST['sort_by']) || isset($_REQUEST['_forum_sort_order'])) {
			$sort_by = (isset($_REQUEST['sort_by'])) ? $_REQUEST['sort_by'] : $default_sort_by;
			$order = (isset($_REQUEST['_forum_sort_order'])) ? $_REQUEST['_forum_sort_order'] : $default_order;

			if (!is_null($user)) {
				// Saves the new style as a preference for the user
				$user->setPreference(Forum::PREFERENCE_SORT_ORDER, $order);
				$user->setPreference(Forum::PREFERENCE_SORT_BY, $sort_by);
			}

		} else if (!is_null($user)) {
			$sort_by = $user->getPreference(Forum::PREFERENCE_SORT_BY);
			$order = $user->getPreference(Forum::PREFERENCE_SORT_ORDER);
			
			if (!$sort_by) $sort_by = $default_sort_by;
			if (!$order) $order = $default_order;
		} else {
			$sort_by = $default_sort_by;
			$order = $default_order;
		}

		$ret = array();
		$ret['sort_by'] = $sort_by;
		$ret['order'] = $order;
		return $ret;
		*/
		return $this->paginator->getSortingOptions();
	}


	/**
	 * Show messages in a nested fashion
	 */
	private function showThreadedMessages(ForumThread $thread) {
		$theme = $this->template;
		$lang = $this->language;
		$url = GFUrl::getUrl();
		$forum = $thread->getForum();
		$forum_id = $forum->getForumId();
		$output = '';

		$msg_array = $thread->getThreadMessagesThreaded();
		
		$output .= '<table border="0" width="100%">';

		if (count($msg_array) > 0) {
			// now show the messages
			$n = count($msg_array);
			for ($i=0; $i < $n; $i++) {
				$message = $msg_array[$i]["message"];
				$level = $msg_array[$i]["level"];
	
				$cell = '<table border="0" width="100%"><tr><td>';
				$cell .= str_repeat('&nbsp', $level*10);
				$cell .= '</td><td width="100%">';
				$cell .= $this->getMessage($message, self::MESSAGE_STYLE_FLAT);
				$cell .= '</td></tr></table>';
				$output .= '<tr><td>'.$cell.'</td></tr>';
			}
			$output .= '</table>';
			$output .= '<br />';
		} else {
			// Wrap the message inside a table in order to be consistent
			//$output .= '<div align="center"><span class="nonefound">'.$lang->getText('ForumThread.NoneFound').'</span></div>';
			$output .= $theme->tableTop(array(''));
			$output .= $theme->tableFullRow('<span class="nonefound">'.$lang->getText('ForumThread.NoneFound').'</span>');
			$output .= $theme->tableBottom($this->getForumButtons($thread->getForum()));
		}
		
//		$output .= '<div align="center"><a href="'.$url->currentUrl(array('_forum_action' => 'ForumThreadBrowse')).'">'.$lang->getText('ForumMessage.BackToThreads').'</a>';			
			$output .='<div align="right">' . HTMLWidget::button("button",$lang->getText('ForumMessage.BackToThreads'),"cancel","","window.location='".$this->buildForumURL(array('action' => 'ForumBrowse', 'forum_id' => $forum_id))."';") . '</div>';
		
		return $output;


	}

	private function getThread(Forum $forum, $msg_array, $show_buttons=true, $selected_message=null) {
		$theme = $this->template;
		$lang = $this->language;
		$url = GFUrl::getUrl();
		
		$ret = '';
		
		$ret .= '<div id="mainTabSet" class="tabber">';

		$ret .= '<div class="tabbertab'.((isset($_GET['selectedtab']) && $_GET['selectedtab']=='thread') ? ' tabbertabdefault' : '') . '" title=" '. $lang->getText('ForumDisplay.ThreadTitle') . '" id="threadtab" >';
			
		$titles = array();
		$titles[] = $lang->getText('ForumDisplay.ThreadTitle');
		$titles[] = $lang->getText('ForumDisplay.AuthorTitle');
		$titles[] = $lang->getText('ForumDisplay.DateTitle');
		
		$ret .= $theme->tableTop($titles);
		
		
		
		$svdt = explode('|',$forum->getSavedDate(Session::getSession()->getUser()));
		$saved_date = (isset($svdt[1]) ? $svdt[1] : $svdt[0]);
		if (count($msg_array) > 0) {
			$this->paginator = new Paginator('forum_thread', -1, count($msg_array), '');			
			$limit = $this->getThreadLimit();
			
			// now show the messages
			$offset = $this->paginator->getOffset();
			for ($i=0; $i < $limit; $i++) {
				$idx = $offset+$i;
				if ($idx >= count($msg_array)) break;
				
				$message = $msg_array[$idx]["message"];
				$level = $msg_array[$idx]["level"];
	
				$cols = array();
				$msg_url = $this->buildForumURL(array('_forum_action' => 'MessageReply', 'message_id' => $message->getForumMessageId()));
				$user = $message->getUser();
				
				$unread = ($saved_date < $message->getPostDate(null));
				
				// if the messge must be selected, display it in bold font instead of as a link
				if (!is_null($selected_message) && ($selected_message->getForumMessageId() == $message->getForumMessageId())) {
					$msg_text = $message->getSubject();
				} else {
					$msg_text = '<a href="'.$msg_url.'">'.$message->getSubject().'</a>';
				}
	
				if ($unread) {
					$cols[] = str_repeat('&nbsp;', ($level*5)).'<strong>'.$msg_text.'</strong>';
				} else {
					$cols[] = str_repeat('&nbsp;', ($level*5)).$msg_text;
				}
				
				if ($user->isUndefinedUser()) {
					$cols[] = $message->getMailSender();
				} else {
					$cols[] = '<a href="'.$url->userUrl($user).'">'.$message->getUser()->getFullName().'</a>';
				}
				$cols[] = $message->getPostDate();
				$ret .= $theme->tableRow($cols);
				
			}
			//$ret .= $theme->listPaginatorRow($this->paginator);
		} else {
			$ret .= $theme->tableFullRow('<span class="nonefound">'.$lang->getText('ForumThread.NoneFound').'</span>');
		}
		
		if ($show_buttons) {
			$ret .= $theme->tableBottom($this->getForumButtons($forum));
		} else {
			$ret .= $theme->tableBottom();
		}
		$ret .='</div>';

		//Disabled for ce
		/*
		if(is_null($forum->getShowAssociationsTab()) || $forum->getShowAssociationsTab()){
			$ret .='<div class="tabbertab' . ((isset($GET['selectedtab']) && $_GET['selectebtab'] == 'associations') ? 'tabbertabdefault' : '') . '" title="' .$lang->getText('Associations') .'" id="associationstab" >';
			$ret .= $this->associationTable($selected_message->getForumMessageId());
			$ret .='</div>';
		}*/
		if(is_null($forum->getShowTagsTab()) || $forum->getShowTagsTab()){
			$ret .='<div class="tabbertab' . ((isset($GET['selectedtab'])  && $_GET['selectedtab'] == 'tags') ?
'tabbertabdefault' : '') . '" title="' . $lang->getText('Tag.TabHeader') . '" id="tagstab" >';
			$ret .= $this->tagsTable($selected_message->getForumMessageId());
			$ret .='</div>';
		}

		
		$ret .='</div>';
		
		if(isset($this->paginator) && is_object($this->paginator)) {
			$ret .= '<div class="paginator">' . $this->paginator->getPaginator() . '</div><br />';
		}
		
		return $ret;
	}

	private function getForumButtons(Forum $forum) {
		$session = Session::getSession();
		$loggedin = $session->isLoggedIn();
		$theme = $this->template;
		$Lang = $this->language;
		$url = GFUrl::getUrl();
		$forum_id = $forum->getForumId();
		
		if ($loggedin && $forum->userCanPost($session->getUser()) ) {
			$cancreate = true;
		} else {
			$cancreate = false;
		}

		$buttons = array();

		if ($cancreate && !$this->readonly) {
			$buttons[$this->buildForumURL(array('_forum_action' => 'ForumThreadAdd', 'forum_id'=>$forum->getForumId()))] = $Lang->getText('Forum.StartThread');
		} else {
			$buttons[$this->buildForumURL(array('_forum_action' => 'ForumThreadAdd', 'forum_id'=>$forum->getForumId()))] = $Lang->getText('Forum.StartThread');
		}
		return $buttons;	

	}
	
	private function getMessageThread($message) {
		$threaded_thread = $message->getForumThread()->getThreadMessagesThreaded();
		$forum = $message->getForumThread()->getForum();
		
		return $this->getThread($forum, $threaded_thread, false, $message);
	}
	
	public function getHistory($forum) {
		$res = '';
		$lang = $this->language;
		$url = GFUrl::getUrl();
		$theme = Session::getSession()->getTheme();

		// Basic validation
		if (!($forum instanceof Forum)) {
			$forum = ForumPeer::retrieveByPk($forum);
			if (is_null($forum)) {
				throw new PermissionDeniedException($Lang->getText('Forum.InvalidForumID'));
			}
		} 
		
		$ref_id = $forum->getRefId();
		$section = $forum->getSection();
		$output = '';
		
		if ($ref_id != $this->ref_id || $section != $this->section) {
			throw new PermissionDeniedException($Lang->getText('ForumThread.InvalidRefId'));
		}
		
		// See if we must show a specific month
		if (isset($_GET['forum_history_month']) && isset($_GET['forum_history_year'])) {
			return $this->getMonthHistory($forum, $_GET['forum_history_month'], $_GET['forum_history_year']);
		}
		
		$history = $forum->getHistory();
		
		// retrieve the month names for quick access
		$month_names = array();
		for ($i=1; $i <= 12; $i++) {
			$month_names[$i] = $lang->getText('Common.Months.'.$i);
		}
		
		$res .= '<p><h3>'.$lang->getText('Forum.History', array($forum->getForumName())).'</h3></p>';
		
		foreach ($history as $year => $months) {
			$res .= '<h2>'.$year.'</h2>';
			
			$cols = array();
			$cols[] = $lang->getText('ForumHistory.MonthTitle');
			$cols[] = $lang->getText('ForumHistory.PostsTitle');
			$res .= $theme->tableTop($cols);
			
			foreach ($months as $month => $posts) {
				$history_url = $this->buildForumURL(array('forum_id' => $forum->getForumId(), 'forum_history_month' => $month, 'forum_history_year' => $year));
				
				$row = array();
				$row[] = '<a href="'.$history_url.'">'.$month_names[$month].'</a>';
				$row[] = $lang->getText('ForumHistory.Posts', array($posts));
				$res .= $theme->tableRow($row);
			}
			$res .= $theme->tableBottom();
			$res .= '<br />'."\n";
		}
		
		return $res;
	}
	
	private function messageEditForm($message_id) {
		$lang = Session::getSession()->getLanguage();
		$message = ForumMessagePeer::retrieveByPk($message_id);
		$forum_id = $message->getForumThread()->getForum()->getForumId();
		$thread_id = $message->getForumThread()->getForumThreadId();
		if (is_null($message) || !$message->userCanUpdate()) {
			throw new PermissionDeniedException();
		}
		
		if (isset($_POST['submitted'])) {
			try {
				$con = Propel::getConnection();
				$message->setSubject( Validator::sanitizeString($_POST['subject']) );
				$message->setBody( Validator::sanitizeHtml($_POST['body']) );
				$message->save();
				
				//
				//      handle the files
				//
				for ($i=0; $i<5; $i++) {
					$error=$_FILES['addfile']['error'][$i];
					$filename=$_FILES['addfile']['name'][$i];
	
					if ($error > 0 && $error != 4) {
						$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
						throw new InvalidDataException($fileerrors);
						continue;
					}
	
	
					$tmp_name=$_FILES['addfile']['tmp_name'][$i];
					if (!is_uploaded_file($tmp_name)) {
						continue;
					}
					$size=$_FILES['addfile']['size'][$i];
					$type=$_FILES['addfile']['type'][$i];
	
					try {
						$fs = new Filesystem();
						$fs->setSection(ForumMessage::SECTION);
						$fs->setRefId($message->getForumMessageId());
	
						$fs->setFileName( $filename );
						//$fs->setFileNameSafe();
						$fs->setFileType( $type );
						$fs->setFileSize( $size );
						$fs->setPostedBy( Session::getSession()->getUser()->getUserId() );
						$fs->save();
						$fs->setFileDataFromPath( $tmp_name );
					} catch (Exception $e) {
						// rollback if we couldn't create a file (write permissions)
						$con->rollback();
						$err = array($e->getMessage());
						throw new InvalidDataException($err);
					}
				}
				if(isset($_POST['deletefile']) && is_array($_POST['deletefile'])){
					foreach ($_POST['deletefile'] as $file_id){
						$message->deleteAttachment($file_id);
					}
				}	
				
	
				$feedback = $lang->getText('ForumMessageEditForm.Edited');
				GFUrl::redirect($this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 
					'thread_id' => $message->getForumThreadId(), 'feedback' => $feedback)));
			} catch (InvalidDataException $ex) {
				$this->errors = $ex->getErrors();
			}
		}
		
		$oFCKeditor = new FCKeditor('body') ;
		$oFCKeditor->BasePath = GFUrl::getUrl()->buildURL('/fckeditor/');
		$oFCKeditor->Value = $message->getBody();
		$oFCKeditor->Width = "700";
		$oFCKeditor->Height = "500";
		$oFCKeditor->ToolbarSet = "GForge";

		$return = '<br />
			<form method="post" enctype="multipart/form-data">
			<input type="hidden" name="submitted" value="1" />
			<p>
				<table border="0" align="center">
				<tr><td align="center">
				<h3>'.$lang->getText('ForumMessage.PostFollowup').'</h3>
				</td></tr>
				<tr><td>
					<strong>'.$lang->getText('ForumMessage.SubjectName').':</strong><br />
					<input type="text" name="subject" value="'.$message->getSubject() .'" size="60" maxlength="90" />
				</td></tr>
				<tr><td>
					<strong>'.$lang->getText('ForumMessage.BodyName').':</strong><br />';
//					<textarea name="body" rows="15" cols="60">'.$message->getBody().'</textarea>
		$return .= $oFCKeditor->CreateHtml();
		$return .= '</td></td>';
		$return .= '</p></table>';

		$return .= '<div id="mainTabSet" class="tabber">';

		$return .= '<div class="tabbertab'.((isset($_GET['selectedtab']) && $_GET['selectedtab']=='attachments') ? ' tabbertabdefault' : '') .'" title="'. $lang->getText('Filesystem.AddNew') . '" id="filestab" ';
		$return .= $this->fileEditForm($message_id, $lang);
		$return .='</div>';
		
		
		$forum = ForumPeer::retrieveByPk($forum_id);
		//Disabled for ce
		/*
		if(is_null($forum->getShowAssociationsTab()) || $forum->getShowAssociationsTab()){
			$return .='<div class="tabbertab' . ((isset($GET['selectedtab']) && $_GET['selectebtab'] == 'associations') ? 'tabbertabdefault' : '') . '" title="' .$lang->getText('Associations') .'" id="associationstab" >';
			$return .= $this->associationTable($message_id);
			$return .='</div>';
		}*/
		
		if(is_null($forum->getShowTagsTab()) || $forum->getShowTagsTab()){
			$return .='<div class="tabbertab' . ((isset($GET['selectedtab'])  && $_GET['selectedtab'] == 'tags') ?
'tabbertabdefault' : '') . '" title="' . $lang->getText('Tag.TabHeader') . '" id="tagstab" >';
			$return .= $this->tagsTable($message_id);
			$return .='</div>';
		}

		$return .='</div>';
		$return .= HTMLWidget::button("submit",$lang->getText('ForumMessage.PostReply'),"forum_post_reply","","");
		$return .= HTMLWidget::button("button",$lang->getText('Common.Back'),"back","","window.location='".$this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 'action' => 'ForumBrowse', 'forum_id' => $forum_id, 'thread_id' => $thread_id))."';");
			
		return $return;
	}

	private function tagsTable($message_id){
		$message = ForumMessagePeer::retrieveByPk($message_id);
		return  TagsRenderer::getTagsTable($message->getForumMessageId(), ForumMessage::SECTION, $this->template, $message->getForumThread()->getForum()->getForumId());
	}

	private function associationTable($message_id){
		$message = ForumMessagePeer::retrieveByPk($message_id);
		$from_section = ForumMessage::SECTION;
		$from_ref_id = $message_id;
		$canmod = $message->userCanUpdate();
		$theme = $this->template;
		$project = GFUrl::getUrl()->getProject();
		
		return renderAssociationList($from_section, $from_ref_id, $canmod, $theme, $project);

	}

		

	private function fileEditForm($message_id){
		
		$message = ForumMessagePeer::retrieveByPk($message_id);		
		$c = new Criteria();
		$c->add(FileSystemPeer::SECTION, ForumMessage::SECTION);
		$c->add(FileSystemPeer::REF_ID, $message_id);
		$files = FileSystemPeer::doSelect($c);	

		$theader[] = $this->language->getText('Filesystem.FileNameName');
		$theader[] = $this->language->getText('Filesystem.FileTypeName');
		$theader[] = $this->language->getText('Filesystem.FileSizeName');
		$theader[] = $this->language->getText('Filesystem.PostedByName');

		if($message->userCanUpdate()){
			$theader[] = $this->language->getText('Delete');
		}
		$html = $this->template->tableTop($theader);
		
		if(count($files) < 1){
			$html .= $this->template->tableFullRow($this->language->getText('Filesystem.NoneFound'));
		} else {
			foreach($files as $file){
				$trow = array();
				$url = GFUrl::getURL()->downloadURL(ForumMessage::SECTION,$message_id,$file->getPrimaryKey(),$file->getFileNameSafe());
				$trow[] = '<a href="'.$url.'">'.$file->getFileName().'</a>';
				$trow[]=$file->getFileType();
				$trow[]=$file->getFileSize();
				$trow[]=$file->getUser()->getUserLink();

				if($message->userCanUpdate()){
					$trow[]='<input type="checkbox" name="deletefile[]" value="'.$file->getFilesystemId().'"/>&nbsp;'.$this->language->getText('Delete');
				}
				$html .= $this->template->tableRow($trow);
			}
		}
		$html .= $this->template->tableBottom();
		$html .= '<table border="0">
			<tr>
			<td><strong>'.$this->language->getText('Filesystem.AddNew') .'</strong><br />
			<input name="addfile[]" size="30" type="file"><br />
			<input name="addfile[]" size="30" type="file"><br />
			<input name="addfile[]" size="30" type="file"><br />
			<input name="addfile[]" size="30" type="file"><br />
			<input name="addfile[]" size="30" type="file">
			</td>
			</tr>
			</table>';
		return $html;

	}



	private function messageDeleteForm($message_id) {
		$lang = Session::getSession()->getLanguage();
		$message = ForumMessagePeer::retrieveByPk($message_id);
		if (is_null($message) || !$message->userCanDelete()) {
			throw new PermissionDeniedException();
		}
		
		if (isset($_POST['submitted'])) {
			if (isset($_POST['sure']) && $_POST['sure'] == 1) {
				// in this case, the whole thread will be deleted
				$thread_deleted = $message->isMainThreadMessage(); 
				$message->delete();
				if ($thread_deleted) {
					$feedback = $lang->getText('ForumThread.Deleted');
					GFUrl::redirect($this->buildForumURL(array('_forum_action' => 'ForumThreadBrowse', 
							'forum_id' => $message->getForumThread()->getForumId(), 'feedback' => $feedback)));
				} else {
					$feedback = $lang->getText('ForumMessageDeleteForm.Deleted');
					GFUrl::redirect($this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 
							'thread_id' => $message->getForumThreadId(), 'feedback' => $feedback)));
				}
			} else {
				$feedback = $lang->getText('ForumMessageDeleteForm.DidntConfirm'); 
				GFUrl::redirect($this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 
						'thread_id' => $message->getForumThreadId(), 'feedback' => $feedback)));
			}
		}
		
		$res = '<p><h4>'.$lang->getText('ForumMessage.DeletePageTitle').': [#'.$message_id.'] "'.$message->getSubject().'"</h4></p>';
		$res .= '<i>'.$lang->getText('ForumMessageDeleteForm.Warning').'</i> <br />';
		$res .= '<form method="post">';
		$res .= '<input type="hidden" name="submitted" value="1" />';
		$res .= '<input type="checkbox" name="sure" value="1" /> '.$lang->getText('ForumMessageDeleteForm.Confirm').'<br /><br />';
		$res .= HTMLWidget::button("submit",$lang->getText('Common.Submit'),"confirm","","");
		$res .= HTMLWidget::button("button",$lang->getText('submit.cancel'),"confirm","","javascript:history.back();");
		$res .= '</form>';
		
		return $res;
	}

	private function getMonthHistory(Forum $forum, $month, $year) {
		$res = '';

		$lang = $this->language;
		$url = GFUrl::getUrl();
		$theme = Session::getSession()->getTheme();

		$msg_array = $forum->getMonthHistory($month, $year);

		$res .= '<p><h3>'.$lang->getText('ForumHistory.HistoryTitle', array($forum->getForumName(), $lang->getText('Common.Months.'.$month), $year)).'</h3></p>';

		$theader = array();
		$theader[] = $lang->getText('ForumMessage.BodyName'); 
		$res .= $theme->tableTop($theader);
		
		if (count($msg_array) > 0){ 
			
			
			$total = count($msg_array);
			$this->paginator = new Paginator('forum_history', -1, $total,'');
			$limit = $this->getThreadLimit();
			
			$offset = $this->paginator->getOffset();
			for ($i=0; $i < $limit; $i++) {
				$idx = $offset+$i;
				if ($idx >= count($msg_array)) break;
				
				$trow = array();
				
				$message = $msg_array[$idx];
				$trow[] = $this->getMessage($message);
				$res .= $theme->tableRow($trow);
			}
			
			$res .= $theme->tableBottom();
			$res .= '<div align="center" class="paginator">'.$this->paginator->getPaginator().'</div><br />';
		} else {
			$res .= $theme->tableFullRow($lang->getText('ForumHistory.NoPosts'));
			$res .= $theme->tableBottom();
		}

	//	$res .= '<br />';
	//	$res .= '<div align="center"><a href="'.$url->currentUrl(array(), array('forum_history_month','forum_history_year','_forum_history_page','_forum_history_total_items')).'">'.$lang->getText('ForumHistory.Back').'</a></div>';
		
		return $res;
	}
	
	private function lockThread($thread_id, $lock) {
		$thread = $this->validateThread($thread_id);
		if ($lock) {
			$thread->setIsLocked(true);
			$feedback = Language::getSessText('ForumThread.LockedFeedback');
		} else {
			$thread->setIsLocked(false);
			$feedback = Language::getSessText('ForumThread.UnlockedFeedback');
		}
		$thread->save();
		
		$action = isset($_GET['return_action']) ? $_GET['return_action'] : 'ForumMessageBrowse';
		GFUrl::redirect($this->buildForumURL(array('feedback' => $feedback, '_forum_action' => $action, 'thread_id' => $thread->getForumThreadId())));
	}
	
	private function setStickyThread($thread_id, $sticky) {
		$thread = $this->validateThread($thread_id);
		if ($sticky) {
			$thread->setIsSticky(true);
			$feedback = Language::getSessText('ForumThread.StickyFeedback');
		} else {
			$thread->setIsSticky(false);
			$feedback = Language::getSessText('ForumThread.UnStickyFeedback');
		}
		$thread->save();

		$action = isset($_GET['return_action']) ? $_GET['return_action'] : 'ForumMessageBrowse';
		GFUrl::redirect($this->buildForumURL(array('feedback' => $feedback, '_forum_action' => $action, 'thread_id' => $thread->getForumThreadId())));
	}

	public function handleBrowse(Forum $forum) {
		$url = GFUrl::getUrl();
		$Lang = Session::getSession()->getLanguage();
		$action = (isset($_GET['_forum_action'])) ? $_GET['_forum_action'] : '';

		switch ($action) {
			case 'ForumThreadAdd': {
				return $this->newThreadForm($forum, $this->buildForumURL(array('_forum_action' => 'ForumThreadBrowse')));
				break;
			}
			case 'ForumMessageBrowse': {
				$c = new Criteria();
				$c->add(ForumThreadPeer::FORUM_THREAD_ID, $_GET['thread_id']);
				$thread = ForumThreadPeer::doSelectJoinForum($c);
				
				if (count($thread) == 0) {
					return $Lang->getText('Forum.InvalidThreadId');
				} else {
					$thread = $thread[0];
					return $this->getThreadMessages($thread);
				}
				break;
			}
			case 'MessageReply': {
				$message = ForumMessagePeer::retrieveByPk($_GET['message_id']);

				if (is_null($message)) {
					return $Lang->getText('Forum.InvalidForumMessageID');
				} else {
					$thread = $message->getForumThread();
					$forum = $message->getForumThread()->getForum();
					$params['thread'] = $thread;
					$form = $this->messageReplyForm($message,
						$this->buildForumURL(array('_forum_action' => 'ForumMessageBrowse', 'thread_id' => $thread->getForumThreadId())));
					if ($this->hasErrors()) {
						$errors = $this->getErrors();
						$params['feedback'] = '* '.implode('<br />* ', $errors);
					}
					
					return $form;
				}
				
				break;
			}
			case 'MessageArchive': {
				return $this->getHistory($_GET['forum_id']);
				break;
			}
			case 'MessageDelete': {
				return $this->messageDeleteForm($_GET['message_id']);
				break;
			}
			case 'MessageEdit': {
				return $this->messageEditForm($_GET['message_id']);
				break;
			}
			case 'LockThread': {
				$this->lockThread($_GET['thread_id'], $_GET['lock']);
				break;
			}
			case 'StickyThread': {
				$this->setStickyThread($_GET['thread_id'], $_GET['sticky']);
				break;
			}
			default: {
				return $this->getThreads($forum);
				break;
			}
		}
	}
	
	/**
	 * Create the URLs for invoking forum actions
	 */
	private function buildForumURL($variables) {
		foreach ($this->kept_variables as $kept_variable) {
			if (array_key_exists($kept_variable, $_GET) && !array_key_exists($kept_variable, $variables)) {
				$variables[$kept_variable] = $_GET[$kept_variable];
			}
		}
		return $this->url->currentBaseURL('', $variables);
	}
	
	public function setReadonly() {
		$this->readonly = true;
	}
	
}

?>

