<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: ProjectScmEditAction.php 947 2006-05-06 20:50:52Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a ProjectScm and saves it to the 
// database
//
require_once('plugins/scm/lib/ProjectScm.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/scmcc/lib/CCRepository.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$Lang = $session->getLanguage();

$project = GFUrl::getUrl()->getProject();
$ccplugin = $project->getPluginByName('scmcc');
if (is_null($ccplugin)) {
	throw new InvalidPluginException('scmcc');
}

$repos = new CCRepository($project);
$ProjectScm = $repos->getScmConfiguration();

// $ProjectScm can be null if SCM is not currently configured for this project
if (!is_object($ProjectScm)) {
	if (!$project->getPermission()->isAdmin()) {
		throw new PermissionDeniedException();
	} else {
		// this means it's the first time we're configuring SCM for this project
		$ProjectScm = new ProjectScm();
		$ProjectScm->setProjectId($project->getProjectId());
	}
} else if (!$ProjectScm->userCanUpdate()) {
	throw new PermissionDeniedException();
}

try {
	//$ProjectScm->setScmServerId( Validator::sanitizeInt($_POST['scm_server_id']) );
	$ProjectScm->setControlled(false);
	$ProjectScm->setHostname( Validator::sanitizeString($_POST['hostname']) );
	$ProjectScm->setRootDirectory( Validator::sanitizeString($_POST['root_dir']) );
	$ProjectScm->setExternalBrowseUrl( Validator::sanitizeString($_POST['external_browse_url']) );
	$ProjectScm->setBrowseText( Validator::sanitizeHtml($_POST['browse_text']) );
	$ProjectScm->setAccessText( Validator::sanitizeHtml($_POST['access_text']) );
	$ProjectScm->setScmServerId(1);		// Hardcoded value for now -- will be implemented in the future
	$ProjectScm->save();
	$feedback = $Lang->getText('ProjectScm.ChangesSaved');
} catch (InvalidDataException $ex) {
	$errors = $ex->getErrors();
	$feedback = '* '.implode('<br />* ', $errors);
}

include('ProjectScmEdit.php');

?>
