<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/tracker/lib/om/BaseTrackerExtraField.php';
require_once("plugin/ActionManager.php");
require_once('lib/html/GFWidget.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Wed Nov 16 20:42:33 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class TrackerExtraField extends BaseTrackerExtraField implements GFWidget {

	const TYPE_SELECT='1';
	const TYPE_CHECKBOX='2';
	const TYPE_RADIO='3';
	const TYPE_TEXT='4';
	const TYPE_MULTISELECT='5';
	const TYPE_TEXTAREA='6';
	const TYPE_STATUS='7';
	const TYPE_RELEASE='8';
	const TYPE_DATE='9';
	const TYPE_NUMERIC='10';
	const TYPE_SERIAL='11';
	const TYPE_MEMBERS='12';
	const INT_TYPES='1,2,3,5,7,8,12';
	const QUERY_TYPES='1,2,3,5,7,8,9,10,11,12';
	const EDITABLE_ELEMENT_TYPES='1,2,3,5,7';
	
	
	const ACTION_CREATE = 1;
	const ACTION_UPDATE = 2;
	const ACTION_DELETE = 3;
	
	protected $keyeddata;
	protected $is_template_field = null;
	protected $old_field_name = null;

	function getId() {
		return $this->getTrackerExtraFieldId();
	}

	function getName() {
		return $this->getFieldName();
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see TrackerExtraField::setDeleted()
	 * @see TrackerExtraField::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}

		// Clean up
		switch ($this->getFieldType()) {
			case self::TYPE_SERIAL:
				$this->deleteSequence($this->getSequenceName());
				break;
		}

		$this->updateTemplate(TrackerExtraField::ACTION_DELETE);
		
		ActionManager::notify("TRACKER_EXTRA_FIELD_DELETE", $this);
		parent::delete($con);
		
	}

	function setFieldName($string) {
		if (is_null($this->old_field_name)) {
			// Must store this for being able to replicate changes to
			// other extra fields
			$this->old_field_name = $this->getFieldName();
		}
		parent::setFieldName($string);

		// build the appropiate alias for this field
		$alias = preg_replace('/[^[:alnum:]]/', '', $string);
		$alias = substr($alias, 0, 20);
		$this->setAlias($alias);

	}

	function setAlias($string) {
		$string = strtolower(StringUtils::removeInvalidChars($string));
		$crit = new Criteria();
		$crit->add(TrackerExtraFieldPeer::TRACKER_ID,$this->getTracker()->getTrackerId());
		$crit->add(TrackerExtraFieldPeer::ALIAS,$string);
		$item=TrackerExtraFieldPeer::doSelect($crit);
		if (count($item) > 0) {
			$string = substr($string, 0, 18);		// Alias must be no more than 20 chars
			$string = $string . rand(10,99);
		}
		parent::setAlias($string);
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if (!$this->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		$errors=array();
		if (strlen($this->field_name) < 3) {
			$errors['field_name'] = Language::getSessText('TrackerExtraField.EmptyName');
		}
		if (!isset($this->field_type)) {
			$errors['field_type'] = Language::getSessText('TrackerExtraField.EmptyFieldType');
		}
		/*
		 else {
			if ($this->field_type == TrackerExtraField::TYPE_TEXTAREA || $this->field_type == TrackerExtraField::TYPE_TEXT) {
				if (!isset($this->attribute1) || !isset($this->attribute2)) {
					$errors['field_dimensions'] =  Language::getSessText('TrackerExtraField.EmptyFieldDimensions');
				}
			}
		}
		*/

		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		$isnew=$this->isNew();
		ActionManager::notify("TRACKER_EXTRA_FIELD_SAVE_BEFORE", $this);
		parent::save($con);
		if ($isnew) {
			$this->createDefaultTrackerItemData();
			$this->updateTemplate(TrackerExtraField::ACTION_CREATE);
			
					
		} else {
			$this->updateTemplate(TrackerExtraField::ACTION_UPDATE);
		}
		ActionManager::notify("TRACKER_EXTRA_FIELD_SAVE", $this);
	}

	/**
	* Adds default data to tracker items
	* Note: This sets the default values for ALL the tracker items. Must be called only when the
	* extra field is created for the first time.
	*/
	public function createDefaultTrackerItemData() {
		$con=Propel::getConnection();
		$stmt=$con->createStatement();
		switch ($this->getFieldType()) {
			case self::TYPE_DATE:
			case self::TYPE_NUMERIC:
			case self::TYPE_TEXTAREA:
			case self::TYPE_TEXT: {
				break;
			}
			case self::TYPE_SERIAL:{
				// For the serial fields, we need to create a corresponding sequence.
				$seqName = $this->getSequenceName();
				
				$con = Propel::getConnection();
				
				$this->createSequence($seqName);
				
				$con->begin();
				try {
					$sql = "INSERT INTO tracker_extra_field_data (tracker_item_id, field_data, tracker_extra_field_id)
							SELECT  tracker_item_id,  nextval('$seqName'), '".$this->getId()."' 
							FROM (SELECT tracker_item_id FROM tracker_item WHERE tracker_id=".$this->getTracker()->getTrackerId()." 
							ORDER BY tracker_item_id ASC) foo";
					$con->createStatement()->executeQuery($sql);
					$con->commit();
				} catch (SQLException $ex) {
					$con->rollback();
					throw $ex;
				}
				break;
			}
			default: {
				$sql="INSERT INTO tracker_extra_field_data (tracker_item_id,field_data,tracker_extra_field_id)
					SELECT tracker_item_id,100,".$this->getTrackerExtraFieldId()."
					FROM tracker_item WHERE tracker_id=".$this->getTracker()->getTrackerId();
				$rs=$stmt->executeQuery($sql);
			}
		}
	}
	
	/**
	 * Sets the default value for this field on the specified tracker item
	 */
	function assignDefaultValue(TrackerItem $tracker_item) {
		$con = Propel::getConnection();
		$stmt = $con->createStatement();

		switch ($this->getFieldType()) {
		case self::TYPE_SERIAL:
			$tefd = new TrackerExtraFieldData();
			$tefd->setTrackerItem($tracker_item);
			$tefd->setTrackerExtraField($this);
			$tefd->setFieldData($this->getSerialNextValue());
			$tefd->save();
			break;
		case self::TYPE_DATE:
		case self::TYPE_TEXTAREA:
		case self::TYPE_RELEASE:
		case self::TYPE_MEMBERS:
			// These fields don't have a default value
			return;
		case self::TYPE_NUMERIC:
		case self::TYPE_TEXT:
			// Just in case
			$stmt->executeQuery('DELETE FROM tracker_extra_field_data WHERE tracker_item_id='.$tracker_item->getTrackerItemId().' AND tracker_extra_field_id='.$this->getPrimaryKey());
			$tefd = new TrackerExtraFieldData();
			$tefd->setTrackerItem($tracker_item);
			$tefd->setTrackerExtraField($this);
			$default_value = $this->getDefaultValue();
			if ($default_value !== "") {
				$tefd->setFieldData($default_value);
			}
			$tefd->save();
			break;
		case self::TYPE_NUMERIC:
		case self::TYPE_TEXT:
		default:
			// Just in case
			$stmt->executeQuery('DELETE FROM tracker_extra_field_data WHERE tracker_item_id='.$tracker_item->getTrackerItemId().' AND tracker_extra_field_id='.$this->getPrimaryKey());
			$tefd = new TrackerExtraFieldData();
			$tefd->setTrackerItem($tracker_item);
			$tefd->setTrackerExtraField($this);
			
			// Get the default selected
			$c = new Criteria();
			$c->add(TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID, $this->getPrimaryKey());
			$c->add(TrackerExtraFieldElementPeer::DEFAULT_SELECTED, true);
			$default_tefe = TrackerExtraFieldElementPeer::doSelectOne($c);
			
			if (!$default_tefe) {
				$tefd->setFieldData(100);
			} else {
				$tefd->setFieldData($default_tefe->getElementId());
			}

			$tefd->save();
		}
	}

	function translateFieldDataToName($data) {
		switch ($this->getFieldType()) {
			case self::TYPE_DATE:
			case self::TYPE_NUMERIC:
			case self::TYPE_SERIAL:
			case self::TYPE_TEXTAREA:
			case self::TYPE_TEXT: {
				return $data;
			}
			default: {
				if ($data == 100) {
					return 'None';
				}
				if (!isset($this->keyeddata)) {
					$els = $this->getTrackerExtraFieldElements();
					foreach ($els as $el) {
						$this->keyeddata[$el->getElementId()]=$el;
					}
				}
				if (isset($this->keyeddata[$data])) {
					if($this->getFieldType() == self::TYPE_MEMBERS){
						$c = new Criteria();
						$c->add(UserPeer::USER_ID, $this->keyeddata[$data]->getElementId());
						$user = UserPeer::doSelectOne($c);
						return $user->getUserlink();
					} else {
						return $this->keyeddata[$data]->getElementName();
					}
				} else {
					return 'None';
				}
			}
		}
	}

	function getTrackerExtraFieldElements() {

		//releases are a hack to select out of an alternative set of tables - the frs tables
		if ($this->getFieldType() == self::TYPE_RELEASE) {
			$sql="select frsr.frs_release_id AS element_id, 
			".$this->getTrackerExtraFieldId()."::int as tracker_extra_field_id, 
			frsp.package_name || '::' || frsr.release_name AS element_name, 1::int as status_id, 
			false::boolean AS default_selected, 0::int AS sort_order, 100::int AS auto_assign_to, false::boolean AS can_commit
			FROM frs_release frsr NATURAL JOIN frs_package frsp 
			WHERE frsp.project_id='".$this->getTracker()->getProject()->getProjectId()."' 
			ORDER BY frsr.release_date ASC, frsp.package_name ASC, frsr.release_name ASC";
			$results=array();
			$con = Propel::getConnection();
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql,ResultSet::FETCHMODE_NUM);
			while($rs->next()) {
				$obj = new TrackerExtraFieldElement();
				$obj->hydrate($rs);
				$results[] = $obj;
			}
			return $results;
			
		} else if ($this->getFieldType() == self::TYPE_MEMBERS){
			$members = $this->getTracker()->getProject()->getMembers();
			$results = array();
			foreach($members as $member){
				$obj = new TrackerExtraFieldElement();
				$obj->setElementName($member->getFirstname() . ' ' . $member->getLastname());
				$obj->setTrackerExtraFieldId($this->getTrackerExtraFieldId());
				$obj->setElementId($member->getUserId());
				$obj->setStatusId(1);
				$obj->setDefaultSelected(false);
				$obj->setSortOrder(0);
				$obj->setAutoAssignTo(100);
				$obj->setCanCommit(false);
				$results[] = $obj;
			}

			return $results;
			
		} else {
			$c = new Criteria();
			$c->addAscendingOrderByColumn(TrackerExtraFieldElementPeer::SORT_ORDER);
			return parent::getTrackerExtraFieldElements($c);
		}
	}
	
	/**
	 * Returns the available values for this item considering the workflow rules
	 */
	function getTrackerExtraFieldElementsWorkflow($current_element) {
		if (!$this->getUseWorkflow()) {
			return $this->getTrackerExtraFieldElements();
		}
		
		if ($current_element == TrackerExtraFieldElement::ELEMENT_NONE) {
			// There's a problem here -- the element is set to 'none' and so there are
			// no possible transitions for this item. There are some situations that
			// can cause this (for example, a field was marked as "uses workflow" when
			// there were already some tracker items). For now, we return all the
			// possible elements
			return $this->getTrackerExtraFieldElements();
		} else {
			// Security check: make sure that $current_element is actually related to this 
			// extra field
			$element = TrackerExtraFieldElementPeer::retrieveByPk($current_element);
			if (!$element || $element->getTrackerExtraFieldId() != $this->getTrackerExtraFieldId()) {
				throw new Exception('Invalid extra field element selected');
			}
		}
		
		// If the user is not logged in, it shouldn't have access to workflow
		if (!Session::getSession()->isLoggedIn()) {
			return array($element);
		}
		
		$allowed_elements = array();
		
		$user = Session::getSession()->getUser();
		$project = $this->getTracker()->getProject();
		
		// Get all the available transitions for this element
		
		if ($project->getParentProjectId() > 0) {
			$projectsql=" IN (".$project->getRootPath().")";
		} else {
			$projectsql="='".$project->getProjectId()."'";
		}

		$sql = 'SELECT DISTINCT * FROM (
					-- Transitions available given the user\'s roles on the project
					SELECT
						tefe.*
					FROM
						tracker_extra_field_element_transition AS tefet,
						tracker_extra_field_element AS tefe,
						tracker_extra_field_element_transition_role AS tefetr
					WHERE
						tefet.element_id='.$current_element.' AND
						tefetr.tracker_extra_field_element_transition_id=tefet.tracker_extra_field_element_transition_id AND
						tefe.element_id=tefet.to_element_id AND
						tefetr.role_id IN (
							-- Get the user\'s roles in the project
							SELECT DISTINCT user_project_role.role_id
							FROM user_project_role NATURAL JOIN user_project
							WHERE (user_project.user_id='.$user->getUserId().' OR user_project.user_id IN 
								(SELECT usergroup_id FROM usergroup_user WHERE member_user_id='.$user->getUserId().'))
								AND user_project.project_id '.$projectsql.'					
						)
				UNION
					-- Must return the current element too
					SELECT * FROM tracker_extra_field_element WHERE element_id='.$current_element.'
				) AS foo
				ORDER BY
					sort_order ASC
				';
				
		$con = Propel::getConnection();
		$stmt = $con->prepareStatement($sql);
		$rs = $stmt->executeQuery(ResultSet::FETCHMODE_NUM);
		
		while ($rs->next()) {
			$element = new TrackerExtraFieldElement();
			$element->hydrate($rs);
			$allowed_elements[] = $element;
		}
		
		return $allowed_elements;
	}
	
	public function isTemplateField() {
		if (is_null($this->is_template_field)) {
			$this->is_template_field = $this->getTracker()->getProject()->getIsTemplate();
		}		
		return $this->is_template_field;
	}
	
	/**
	 * This function replicates the changes across the projects that have this project
	 * as template
	 */	
	private function updateTemplate($action) {
		// Replicate the data only if it's a field on a template project and it's a
		// site admin
		if (!$this->isTemplateField() || !Session::getSession()->isSiteAdmin()) {
			return;
		}
		
		switch ($action) {
		case TrackerExtraField::ACTION_CREATE:
			// Get a list of all the trackers that have to be updated
			$c = new Criteria();
			$c->add(TrackerPeer::TRACKER_NAME, $this->getTracker()->getTrackerName());
			$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
			$c->add(ProjectPeer::TEMPLATE_PROJECT_ID, $this->getTracker()->getProjectId());
			$trackers = TrackerPeer::doSelect($c);
			foreach ($trackers as $tracker) {
				if ($tracker->getTrackerId() == $this->getTracker()->getTrackerId()) {
					// Shouldn't happen, means that the project.template_project_id=project.project_id
					// but we avoid infinite loop anyways
					continue;
				}
				$new_tracker_extra_field = $this->copy();
				$new_tracker_extra_field->setTracker($tracker);
				$new_tracker_extra_field->save();
			}
			
			break;
		case TrackerExtraField::ACTION_UPDATE:
			// Get the list of fields that must be updated
			if (!is_null($this->old_field_name)) {
				$old_field_name = $this->old_field_name;
			} else {
				// This means the field name wasn't changed
				$old_field_name = $this->getFieldName();
			}
			$c = new Criteria();
			$c->add(TrackerExtraFieldPeer::FIELD_NAME, $old_field_name);
			$c->addJoin(TrackerExtraFieldPeer::TRACKER_ID, TrackerPeer::TRACKER_ID);
			$c->add(TrackerPeer::TRACKER_NAME, $this->getTracker()->getTrackerName());
			$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
			$c->add(ProjectPeer::TEMPLATE_PROJECT_ID, $this->getTracker()->getProjectId());
			$extra_fields = TrackerExtraFieldPeer::doSelect($c);
			foreach ($extra_fields as $extra_field) {
				if ($extra_field->getTrackerExtraFieldId() == $this->getTrackerExtraFieldId()) {
					continue;
				}
				
				$extra_field->setFieldName($this->getFieldName());
				$extra_field->setAttribute1($this->getAttribute1());
				$extra_field->setAttribute2($this->getAttribute2());
				$extra_field->setIsRequired($this->getIsRequired());
				$extra_field->setFieldOrder($this->getFieldOrder());
				$extra_field->setShowOnSubmit($this->getShowOnSubmit());
				$extra_field->setUseInQuickBrowse($this->getUseInQuickBrowse());
				$extra_field->save();
			}
			
			break;
		case TrackerExtraField::ACTION_DELETE:
			// Get the list of fields that must be deleted
			$c = new Criteria();
			$c->add(TrackerExtraFieldPeer::FIELD_NAME, $this->getFieldName());
			// Extra safety measure -- shouldn't be necessary but it doesn't hurt
			$c->add(TrackerExtraFieldPeer::FIELD_TYPE, $this->getFieldType());
			$c->addJoin(TrackerExtraFieldPeer::TRACKER_ID, TrackerPeer::TRACKER_ID);
			$c->add(TrackerPeer::TRACKER_NAME, $this->getTracker()->getTrackerName());
			$c->addJoin(TrackerPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
			$c->add(ProjectPeer::TEMPLATE_PROJECT_ID, $this->getTracker()->getProjectId());
			$extra_fields = TrackerExtraFieldPeer::doSelect($c);
			foreach ($extra_fields as $extra_field) {
				if ($extra_field->getTrackerExtraFieldId() == $this->getTrackerExtraFieldId()) {
					continue;
				}
				$extra_field->delete();
			}
			break;
		}
	}
	
	function setDefaultValue($value) {
		$type = $this->getFieldType();
		if ($type != self::TYPE_TEXT && $type != self::TYPE_NUMERIC) {
			// This must not be set for other types of fields
			$value = '';
		}
		
		if ($type == self::TYPE_NUMERIC && !empty($value) && !is_numeric($value)) {
			$value = '';
		}

		parent::setDefaultValue($value);
	}
	
	/**
	 * Gets the next serial value for a serial extra field
	 * @param	bool $autoCreate	Wheter the sequence should be automatcally created if it doesn't exist.
	 */	
	public function getSerialNextValue($autoCreate = true) {
		$con = Propel::getConnection();
		$seqName = $this->getSequenceName();
		$idGen = $con->getIdGenerator();
		
		try {
			$nextval = $idGen->getId($seqName);
			return $nextval;
		} catch (SQLException $ex) {
			// Exception thrown if the sequence doesn't exist. Try to create it.
			if (!$autoCreate) {
				throw $ex;
			}
			
			$this->createSequence($seqName);
			return $this->getSerialNextValue(false);
		}
	}
	
	/**
	 * Returns the corresponding sequence associated to this extra field (only if type == TYPE_SERIAL)
	 */
	private function getSequenceName() {
		return 'tef_'.$this->getTrackerExtraFieldId().'_seq';
	}
	
	private function createSequence($seqName) {
		try {
			$sql = "CREATE SEQUENCE $seqName";
			Propel::getConnection()->createStatement()->executeQuery($sql);
		} catch (SQLException $ex) {
			// The sequence already exists. Ignore the error.
		}
	}
	
	private function deleteSequence($seqName) {
		try {
			$sql = "DROP SEQUENCE $seqName";
			Propel::getConnection()->createStatement()->executeQuery($sql);
		} catch (SQLException $ex) {
			// The sequence didn't exist. Ignore the error.
		}
	}
	
		
}
?>
