<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. P�ez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/plugin/GFProjectPlugin.php');
require_once('plugin/ActionManager.php');
Require_once('SystemUtils.php');
require_once('lib/html/GFWidget.php');
require_once('plugins/wiki/lib/WikiPagePeer.php');
require_once('plugins/wiki/lib/WikiProjectSetting.php');
require_once('plugins/wiki/lib/Wiki.php');
require_once('lib/GFExportImportUtils.php');

class WikiPlugin implements GFProjectPlugin {

	protected $project;

	CONST NOACCESS = 0;
	CONST READ = 1;
	CONST EDIT = 2;
	CONST ADMIN = 3;
	CONST NOANON = 0;
	CONST ANON = 1;
	CONST IS_PRIVATE = 0;
	CONST IS_PUBLIC = 1;	
	CONST SECTION = "wiki";
	
	function __construct(/*$project*/) {
		//$this->project = $project;
	}
	

	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	function initialize() {
		$listenTo = array(
			'BUILD_PROJECT_TABS',
	//		'BUILD_PROJECT_SUMMARY',
			'GET_WIKI_NAV_ELEMENTS',
			'PROJECT_APPROVE',
			'BUILD_MYPAGE_ELEMENTS',
			'BUILD_MYPAGE_TABS',
			'BUILD_USER_TABS',
			'GET_ROLE_CHOICES',
			'UPDATE_ROLE_CHOICES',
			'GET_OBSERVER_ROLE_CHOICES',
			'UPDATE_OBSERVER_SETTINGS',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'RENDER_ASSOCIATION_LINK',
			'GET_ASSOCIATED_ITEM_INFO',
			'ASSOCIATION_SECTION',
			'ASSOCIATION_CHECK',
			'GET_DISK_USAGE',
			'MONITOR_CHECK_PERMISSIONS',
			'BUILD_MYPAGE_MONITORS',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'CHECK_DOWNLOAD_PERMS',
			'BUILD_RSS_FEED',
			'SCM_COMMIT',
			'USER_PROJECT_DELETE',
			'USERGROUP_USER_REMOVE',
			'GET_PROJECT_SETTINGS_FORM',
			'SAVE_PROJECT_SETTINGS',
			'DO_TAG_SEARCH'
			);
		ActionManager::registerListeners($this, $listenTo);
	}

	private function _buildMainTabsListener($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}		
		$project = $params['project'];

		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Wiki.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','wiki'),
				'id'	=> 'wiki'
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getWikiNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getWikiNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Wiki Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Wiki Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Wiki Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Wiki Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 * /
	private function _buildProjectSummary($params) {
		$result = new PluginResult($this);

		// TODO (entire method)
		$project = $params['project'];
		$theme = $params['theme'];
		
		//$value = $theme->infoTableTop(Language::getSessText('Project.Wikis'));
		
		$value = '<hr size="1"/><a href="'. GFUrl::getUrl()->projectURL($project,'','wiki').'">';
		$value .= $theme->image('ic/forum20g.png','20','20',array('alt'=>Language::getSessText('Project.ShortWiki')));
		$value .= Language::getSessText('Project.Wiki').'</a>';
		//$value .= $theme->infoTableBottom();
			
		$result->setResult($value);
		return $result;	
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}

	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	function getName() {
		return 'wiki';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	function getListeners() {
		return array(); //XXX
	}

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed

	 *	@return PluginResult
	 */
	function handleAction($listenerName,$params) {
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			/*case 'BUILD_PROJECT_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			} */
			case 'GET_WIKI_NAV_ELEMENTS': {
				return $this->getForumNavElements();
				break;
			}
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}			
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'GET_ROLE_CHOICES': {
				return $this->getRoleChoices($params);
				break;
			}
			case 'UPDATE_ROLE_CHOICES': {
				return $this->updateRoleChoices($params);
				break;
			}
			case 'GET_OBSERVER_ROLE_CHOICES': {
				return $this->getObserverRoleChoices($params);
				break;
			}
			case 'UPDATE_OBSERVER_SETTINGS': {
				return $this->updateObserverSettings($params);
				break;
			}
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}
			case 'RENDER_ASSOCIATION_LINK': {
				return $this->renderAssociationLink($params);
				break;
			}
			case 'GET_ASSOCIATED_ITEM_INFO': {
				return $this->getAssociatedItemInfo($params);
				break;
			}
			case 'ASSOCIATION_SECTION': {
				return $this->getAssociationSection($params);
				break;
			}
			case 'ASSOCIATION_CHECK': {
				return $this->getAssociationCheck($params);
				break;
			}
			case 'GET_DISK_USAGE': {
				return $this->getDiskUsage($params);
				break;
			}
			case 'MONITOR_CHECK_PERMISSIONS': {
				return $this->monitorCheckPermissions($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
			case 'CHECK_DOWNLOAD_PERMS': {
				return $this->checkDownloadPerms($params);
				break;
			}
			case 'BUILD_RSS_FEED': {
				return $this->buildRssFeed($params);
				break;
			}
			case 'SCM_COMMIT': {
				return $this->scmCommit($params);
				break;
			}
			case 'USER_PROJECT_DELETE': {
				return $this->deleteUserMonitors($params);
				break;
			}
			case 'USERGROUP_USER_REMOVE': {
				return $this->deleteUsergroupUserMonitors($params);
				break;
			}
			case 'DO_TAG_SEARCH': {
				return $this->getSearchByTagsResult($params);
				break;
			}

			case 'GET_PROJECT_SETTINGS_FORM': {
				return $this->getProjectSettingsForm($params);
				break;
			}
			case 'SAVE_PROJECT_SETTINGS': {
				return $this->saveProjectSettings($params);
				break;
			}
		}
	}



	private function deleteUsergroupUserMonitors($usergroup_user) {
		$con = Propel::getConnection();		

		$project_ids = $usergroup_user->getProjectMonitorsToDelete();
		$user_id = $usergroup_user->getMemberUserId();

		if ($project_ids != '') {
			$stmt = $con->createStatement();
			//Delete monitor for the projects
			$sql = "DELETE FROM monitor WHERE 
							(
								(section='".WikiPlugin::SECTION."' AND ref_id IN
									(
										SELECT project.project_id FROM project, wiki_project_setting WHERE
									  wiki_project_setting.project_id = project.project_id
									  AND
										section='project'
										AND
									 	project.project_id IN (".$project_ids.")
									 	AND
									 	NOT (project.is_public = true AND wiki_project_setting.is_public = true)
									)
								)
							OR
								(section='wikipage' AND ref_id IN 
									(
										SELECT wiki_page_id FROM wiki_page, project, wiki_project_setting WHERE
									  wiki_page.ref_id = project.project_id
									  AND
									  wiki_project_setting.project_id = project.project_id
									  AND
										section='project'
										AND
									 	wiki_page.ref_id IN (".$project_ids.")
									 	AND
									 	NOT (project.is_public = true AND wiki_project_setting.is_public = true)
									 )
								)
							)
							AND
							user_id = ".$user_id;
			$stmt->executeQuery($sql);
		}		
	}


	private function deleteUserMonitors($user_project) {
		$user_id = $user_project->getUserId();
		$project = $user_project->getProject();
		if (is_object($project)) {
			if (!$project->isPublic()) {
				$con = Propel::getConnection();
				$stmt = $con->createStatement();
		
				$sql = "DELETE FROM monitor WHERE 
								(
									(section='".WikiPlugin::SECTION."' AND ref_id =".$project->getProjectId().")
								OR
									(section='".WikiPagePeer::SECTION."' AND ref_id IN 
										(SELECT wiki_page_id as ref_id FROM wiki_page WHERE section='project' AND
										 wiki_page.ref_id = ".$project->getProjectId()."))
								)
								AND
									user_id = ".$user_id;
		
				$stmt->executeQuery($sql);
			}
		}
	}


	private function getAssociationCheck($params) {
		if ($params->getSection() == WikiPage::SECTION) {
			$page=WikiPagePeer::retrieveByPk($params->getRefId());
			if (!is_object($page) || !$page->userCanUpdate()) {
				throw new InvalidDataException(array('The wiki page doesn\'t exist or you don\'t have permissions for it'));
			}
		} 
		if ($params->getToSection() == WikiPage::SECTION) {
			$page=WikiPagePeer::retrieveByPk($params->getToRefId());
			if (!is_object($page) || !$page->userCanUpdate()) {
				throw new InvalidDataException(array('The wiki page doesn\'t exist or you don\'t have permissions for it'));
			}
		}
	}

	private function getAssociationSection($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$result->setResult(array(WikiPage::SECTION,'Wiki Page'));
		return $result;
	}

	private function renderAssociationLink($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==WikiPage::SECTION) {
				$page=WikiPagePeer::retrieveByPk($param[1]);
				if (is_object($page)) {
					$ret[]=array('Wiki Page: '.$page->getWikiPageLink(true),
					$param[2],
					$param[3]);
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}

	/**
	 *	Returns additional info for the associated item
	 *		 
	 *	@param array $params needed params to build the associated item info
	 * 	@return array with associated item info (Description, URL)
	 */	 			
	private function getAssociatedItemInfo($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==WikiPage::SECTION) {
				$page=WikiPagePeer::retrieveByPk($param[1]);
				if (is_object($page)) {
					$ret[]=array(
					$param[2],
					$param[3],
					$param[4],
					$param[5],
					$param[6],			
					'Wiki Page: '.$page->getPageName(),
					$page->getWikiPageLink(false));
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}
	
	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	
		
	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','wiki');
	}

	/**
	 * 	The addMember listener - this hook will fire
	 *	whenever a member is added to a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function addMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The removeMember listener - this hook will fire
	 *	whenever a member is removed from a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function removeMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The approveProject listener - this hook will fire
	 *	when a project is approved.
	 *	@param	Project	the project object
	 *	@return	boolean
	 */
	function approveProjectListener($params) {
		return false; // XXX
	}

	/**
	 * 	The deleteProject listener - this hook will fire
	 *	when a project is deleted.
	 *	@return	boolean
	 */
	function deleteProjectListener() {
		return false; // XXX
	}

	/**
	 * 	The checkPermission - this listener will fire
	 *	when permissions are being checked for this project.
	 *	@param	Array	An associative array containing 
	 *	'section_name' a name like 'tracker' or 'tracker_item'
	 *	'section_id'=>id (the id of the section, for example a specific tracker)
	 *	'subsection_id'=>id (the id of the subsection, for example an tracker_item id)
	 *	'perm_level'=>int (the value, such a 1 for read, 2 for write, that is being requested)
	 *	@return	boolean
	 * /
	function checkPermission($params);
*/

	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}
	
	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Wiki.NoAccess'); //descriptions
		$return[1][] = WikiPlugin::NOACCESS; // values
		$return[0][] = Language::getSessText('Wiki.Read');
		$return[1][] = WikiPlugin::READ;
		$return[0][] = Language::getSessText('Wiki.Edit');
		$return[1][] = WikiPlugin::EDIT;
		$return[0][] = Language::getSessText('Wiki.Admin');
		$return[1][] = WikiPlugin::ADMIN;
		
		return $return;
	}

	/**
	 * returns the possible site-wide admin role settings for this plugin
	 *
	 * @return array	the roles
	 */	
	function getAdminRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Forum.None'); //descriptions
		$return[1][] = ForumPlugin::NOACCESS; // values
		$return[0][] = Language::getSessText('Forum.Admin');
		$return[1][] = ForumPlugin::ADMIN;
		return $return;
	}

	/**
	 * 	The getRoleChoices listener - this hook will fire
	 *	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings or null
	 *	@return	array
	 */
	function getRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$roles = array();
		$roles[0]['ref_id'] = 0;
		$roles[0]['section'] = 'Wiki';
		$roles[0]['description'] = Language::getSessText('Wiki.Roles.Description');
		$roles[0]['subsection'] = '-';
		$roles[0]['settings'][0] = 'wiki'; // the section
		$roles[0]['settings'][] = $this->getAvailableRoleSettings();

		$result = new PluginResult($this);
		$result->setResult($roles);
		return $result;
	}

	/**
	 * returns the settings for the wiki posting
	 *
	 * @return array
	 */
	function getPostingObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Wiki.NoAnon'); //descriptions
		$return[1][] = WikiPlugin::NOANON; // values
		$return[0][] = Language::getSessText('Wiki.Anon');
		$return[1][] = WikiPlugin::ANON;
		return $return;		
	}

	/**
	 * returns the settings for the wiki status
	 *
	 * @return array
	 */
	function getStatusObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Wiki.Public');
		$return[1][] = WikiPlugin::IS_PUBLIC;
		$return[0][] = Language::getSessText('Wiki.Private'); //descriptions
		$return[1][] = WikiPlugin::IS_PRIVATE; // values		
		return $return;		
	}	
	
	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$choices = array();
		$i = 0;
		$key = $params['project']->getProjectId();
		$choices[$i]['ref_id'] = $key;
		$choices[$i]['section'] = Language::getSessText('Wiki.Privileges');
		$choices[$i]['description'] = Language::getSessText('Wiki.Roles.Description');
		$choices[$i]['subsection'] = '-';
		$choices[$i]['settings'][0] = 'wikipublic';
		$choices[$i]['settings'][] = $this->getStatusObserverSettings();
		//TODO: IMPLEMENT THIS OPTION
/*		$i++;
		$choices[$i]['ref_id'] = $key;
		$choices[$i]['section'] = Language::getSessText('Wiki.Privileges');
		$choices[$i]['subsection'] = '-';
		$choices[$i]['settings'][0] = 'wikianon';
		$choices[$i]['settings'][] = $this->getPostingObserverSettings();
*/
		$result = new PluginResult($this);
		$result->setResult($choices);

		return $result;

	}
	
	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */
	function updateObserverSettings($params) {
		$project = $params['project'];
		$data = $params['data'];
		
		foreach ($data as $section => $value) {
			if ($section != 'wikipublic') {
				continue;
			}
			
			// $value is just an array with project_id => observer_setting
			$k = array_keys($value);
			$project_id = $k[0];
			$observer_setting = $value[$project_id];
			
			$c = new Criteria();
			$c->add(WikiProjectSettingPeer::PROJECT_ID, $project_id);
			$setting = WikiProjectSettingPeer::doSelectOne($c);
			if (!$setting) {
				$setting = new WikiProjectSetting();
				$setting->setProjectId($project_id);
			}
			$setting->setIsPublic($observer_setting);
			$setting->save();
		}
	}
	
	/**
	 * Retrieve the corresponding setting (value) for the ref_id passed as argument, using the appropriate section
	 *
	 * @param int $ref_id
	 * @param string $section
	 * @return setting
	 */
	function getObserverSetting($ref_id,$section) {
		switch ($section) {
			case 'wikipublic' : {
				$setting = WikiProjectSettingPeer::retrieveByPk($ref_id);
				if (!$setting) {
					// Record is not on the database -- assume it's public by default
					return WikiPlugin::IS_PUBLIC;
				}
				return $setting->getIsPublic();
			}
			default : {
				return '';
			}
		}
	}
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		// Copy the wiki pages from the template project
		$template_project = $params['template_project'];
		$project = $params['project'];
		
		if (!$template_project->usesPlugin('wiki')) {
			return;
		}
		
		if (!$template_project) {
			return;
		}
		
		// Copy the project settings
		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		$con->createStatement()->executeQuery('DELETE FROM wiki_project_setting WHERE project_id='.$project->getProjectId());
		$con->createStatement()->executeQuery('INSERT INTO wiki_project_setting(project_id, is_public, wiki_type, show_associations_tab, show_tags_tab) 
							SELECT '.$project->getProjectId().', is_public, wiki_type, show_associations_tab, show_tags_tab
							FROM wiki_project_setting WHERE project_id='.$template_project->getProjectId()
						);			
		
		// Copy the pages
		$c = new Criteria();
		$c->add(WikiPagePeer::SECTION, 'project');
		$c->add(WikiPagePeer::REF_ID, $template_project->getProjectId());
		$template_pages = WikiPagePeer::doSelect($c);
		
		if (!empty($template_pages)) {
			foreach ($template_pages as $template_page) {
				$page = $template_page->copy();
				$page->setRefId($project->getProjectId());
				$page->setLatestVersion(1);
				$page->save();
				
				// Copy the latest version
				$template_version = $template_page->getLatestVersion();
				$version = new WikiVersion();
				$version->setWikiPageId($page->getWikiPageId());
				$version->setPageVersion(1);
				$version->setPostedBy(User::UNDEFINED_USER_ID);
				$version->save();
				
				// Create the filesystem object
				$fs = new Filesystem();
				$fs->setFileName($page->getPageName());
				$fs->setPostedBy(User::UNDEFINED_USER_ID);
				$fs->setDeleted(false);
				$fs->setFileType('text');
				$fs->setRefId($version->getPrimaryKey());
				$fs->setSection(WikiPlugin::SECTION);
				$fs->save();
				$fs->setFileDataFromString($template_page->getLatestContents(), true);
			}
		} else {
			// For some reason the template project had the wiki plugin but it didn't have
			// any page. Generate the default pages.
			try {
				$wiki = new Wiki('project', $project->getProjectId(), null, $project);
				$wiki->createNewPage('FrontPage',Language::getSessText('Wiki.Defaults.FrontPage'));
				$wiki->createNewPage('Wiki',Language::getSessText('Wiki.Defaults.Wiki'));
				$wiki->createNewPage('HowToUseWiki',Language::getSessText('Wiki.Defaults.HowToUseWiki'));
				$wiki->createNewPage('AddingPages',Language::getSessText('Wiki.Defaults.AddingPages'));
				$wiki->createNewPage('SandBox',Language::getSessText('Wiki.Defaults.SandBox'));
				$wiki->createNewPage('RecentVisitors',Language::getSessText('Wiki.Defaults.RecentVisitors'));
				$wiki->createNewPage('MostPopular',Language::getSessText('Wiki.Defaults.MostPopular'));
				$wiki->updateStatistics();
			} catch (Exception $ex) {
				// Do nothing
			}
		}
	}
	
	private function updateRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$section = self::SECTION;

		$project = $params['project'];
		$data = $params['data'];
		$role = $params['role'];
		
		if (!isset($data[$section])) {
			return;
		}
		
		foreach ($data[$section] as $ref_id => $value) {
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
			$c->add(RoleSettingPeer::SECTION, $section);
			$c->add(RoleSettingPeer::REF_ID, 0);
			$rolesetting = RoleSettingPeer::doSelectOne($c);
			if (!is_object($rolesetting)) {
				// Enter here if we have a role setting which we didn't have previously (e.g.: activate an inactive plugin)
				$rolesetting = new RoleSetting();
				$rolesetting->setSection($section);
				$rolesetting->setRefId(0);
				$rolesetting->setRoleId($role->getPrimaryKey());
			}
			
			// Being a project admin overrides any other value
			if (isset($data['projectadmin']) && $data['projectadmin'][0] >= Project::PROJECT_ROLE_ADMIN) {
				$value = self::ADMIN;
			}			
			
			$rolesetting->setValue($value);
			$rolesetting->save();						
		}	
		
		return;
	}
	
	/**
	 * returns the options to print in the combobox
	 *
	 * @param unknown_type $params
	 * @return unknown
	 */
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		$value = array();
		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('Wiki.ProjectWiki');
		} else {
			$text = Language::getSessText('Wiki.AllProjectsWiki');
		}
		$value[] = array(
				'value'	=> $this->getName(),
				'display'	=> $text,
			);
		
		$result->setResult($value);
		return $result;		
	}
	
	/**
	 * returns an array of the fields where we want the search to be done upon
	 *
	 */
	private function getSearchFields() {
		return array(WikiPagePeer::PAGENAME);
	}

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return PluginResult
	 */
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;
		}
	}	


	private function getSearchByTagsResult($params) {
		
		$result = new PluginResult($this);
		$search = new GFSearch($params);

		$result->setResult($this->searchByTags($search));

		return $result;

	}

	/**
	 *      return an sql statement for calculating disk usage
	 */
	function getDiskUsage($params) {
		$sql="
			SELECT project_id, SUM(usage) AS usage FROM (
				SELECT wp.ref_id AS project_id, sum(fs.file_size+length(fsi.vectors)) AS usage
				FROM wiki_page wp JOIN wiki_version wv USING (wiki_page_id),
					filesystem fs JOIN filesystem_idx fsi USING (filesystem_id), project pr
				WHERE wp.ref_id=pr.project_id AND wv.wiki_version_id=fs.ref_id
					AND wp.section='project' AND fs.section='wiki' 
				GROUP BY wp.ref_id
				UNION
				SELECT wp.ref_id AS project_id, sum(fs.file_size+length(fsi.vectors)) AS usage
				FROM wiki_page wp, filesystem fs JOIN filesystem_idx fsi USING (filesystem_id)
				WHERE fs.section='wikifiles' AND fs.ref_id=wp.wiki_page_id
					AND wp.section='project'
				GROUP BY wp.ref_id
			) AS u
			GROUP BY project_id";
		$result = new PluginResult($this);
		$result->setResult($sql);
		return $result;
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			$ref_id = null;
			$project = null;
		} else {
			$ref_id = $search->getProjectId();
			$project = ProjectPeer::retrieveByPk($ref_id);
		}
		
		$section = 'project';
		$theme = Session::getSession()->getTheme();
		
		$wiki = new Wiki($section,$ref_id,$theme,$project);
		
		return $wiki->getSearchResults($search);
	}


	function searchByTags(GFSearch $search){
		$words = $search->getWordsForQuery();
		$project_id = $search->getProjectId();
		$template = $search->getTemplate();

		//Clean key words from tsearch operators
		$words = preg_replace('/,+/',',', $words);
		$tsearch_operators = array('(', ')', '!', '|','&');
		$replacements = array('','','','',',');
		$words = str_replace($tsearch_operators, $replacements, $words);
		$words_arr = explode(',', $words);

		$wc = count($words_arr);
		if($wc == 0){
			return null;
		}

		for($i = 0; $i < $wc; $i++){
			$words_arr[$i] = "'" . $words_arr[$i] . "'";
		}
		$words = implode(',', $words_arr);

		if($project_id) {
			//Project search


			//Check tag settings 
			$settings = WikiProjectSettingPeer::retrieveByPk($project_id);
			if($settings && !$settings->getShowTagsTab()){
				return null;
			}


			$project = ProjectPeer::retrieveByPk($project_id);

			if($project->usesPlugin('wiki')){
				$wiki_prj =" AND wiki_page.ref_id = $project_id ";
			
				$wiki_page_sql = "SELECT  'wikipage' AS section, wiki_page.wiki_page_id AS ref_id,
				wiki_page.pagename AS description, project.unix_name AS prj_unix_name
				FROM wiki_page INNER JOIN project ON wiki_page.ref_id = project.project_id
				WHERE wiki_page.wiki_page_id IN
				(SELECT tag_entry.ref_id FROM tag_entry WHERE section='wikipage'
				AND tag_name IN ($words)
				GROUP BY tag_entry.ref_id
				HAVING COUNT(tag_entry.ref_id) = $wc) AND wiki_page.section='project' $wiki_prj";

				$con = Propel::getConnection();
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($wiki_page_sql, ResultSet::FETCHMODE_ASSOC);
		

				//Test if we can access the wiki
				
				if($rs->getRecordCount() > 0){
					$rs->next();
					$wp = WikiPagePeer::retrieveByPk($rs->getInt('ref_id'));
					//If we can access a page, then we can access all of them
					if($wp->userCanRead()){
						$rs->previous();
					} else {
						//User doesn't have read access
						return null;
					}
				} else {
					//Result set is empty
					return null;
				}


			} else {
				return null;
			}

		} else {
			//Global search

			$wiki_perms_sql = "SELECT DISTINCT wiki_page.ref_id AS project_id
			FROM wiki_page WHERE wiki_page.section='project' AND
			wiki_page.wiki_page_id IN
			(SELECT tag_entry.ref_id FROM tag_entry WHERE
			section='wikipage'
			AND tag_name IN ($words)
			GROUP BY tag_entry.ref_id
			HAVING COUNT(tag_entry.ref_id) = $wc)";
	
			$con = Propel::getConnection();
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($wiki_perms_sql, ResultSet::FETCHMODE_NUM);
		

			$wiki_aux = '';
			if($rs->getRecordCount() > 0) {
				while($rs->next()){
					$project_ids[] = $rs->getInt(1);
				}
				$projects = ProjectPeer::retrieveByPks($project_ids);

				$project_ids = array();
				foreach($projects as $project){
					if($project->usesPlugin('wiki')){
						$c = new Criteria();
						$c->add(WikiPagePeer::REF_ID, $project->getProjectId());
						$wp = WikiPagePeer::doSelectOne($c);

						$setting = WikiProjectSettingPeer::retrieveByPk($project->getProjectId());
						//If we can read one page, we can read all
						if($wp->userCanRead() && (!$setting || $setting->getShowTagsTab())){
							$project_ids[] = $project->getProjectId();
						}
					}
				}

				if(count($project_ids) == 0){
					return null;
				} else {
					$wiki_aux = ' AND wiki_page.ref_id IN ( ' . implode(',', $project_ids) .')';
				}
			}

			$wiki_page_sql = "SELECT  'wikipage' AS section, wiki_page.wiki_page_id AS ref_id,
			wiki_page.pagename AS description, project.unix_name AS prj_unix_name,
			project.project_name AS prj_name
			FROM wiki_page INNER JOIN project ON wiki_page.ref_id = project.project_id
			WHERE wiki_page.wiki_page_id IN
			(SELECT tag_entry.ref_id FROM tag_entry WHERE section='wikipage'
			AND tag_name IN ($words)
			GROUP BY tag_entry.ref_id
			HAVING COUNT(tag_entry.ref_id) = $wc) AND wiki_page.section='project' $wiki_aux";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($wiki_page_sql, ResultSet::FETCHMODE_ASSOC);

		}

		$ret = null;
		if($rs->getRecordCount() > 0){
			$Lang = Session::getSession()->getLanguage();

			while($rs->next()){
				$trow = array();
				if(!$project_id){
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name')) . '">'. $rs->getString('prj_name') . '</a>';
				}

				$trow[] = $rs->getInt('ref_id');
				$trow[] = $Lang->getText('Tag.WikiPageSection');
				$trow[] = '-';
				$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), '', 'wiki','',array('pagename'=>$rs->getString('description'))) .'">' . $rs->getString('description') . '</a>';

				$ret .= $template->tableRow($trow);

			}
		}

		return $ret;

	}

	/**
	 * Auxiliar function to check if a user is monitoring the project's wiki
	 */
	public function userIsMonitoring(User $user, Project $project) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $project->getProjectId());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		
		if (MonitorPeer::doCount($c) > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Check if the user can monitor certain item
	 */	
	private function monitorCheckPermissions($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		
		if ($section != self::SECTION && $section != WikiPage::SECTION) {
			return;
		}
		$pr = new PluginResult($this);
		
		//check for monitoring of entire wiki
		if ($section == self::SECTION) {
			$wps = WikiProjectSettingPeer::retrieveByPk($ref_id);
			if (!$wps) {
				$result = false;
			} elseif ($wps->getIsPublic() && $wps->getProject()->getIsPublic()) {
				//anyone can monitor if it's a public wiki and project is public
				$result = true;
			} else {
				$perm = $wps->getProject()->getPermission();
				$result = $perm->check(self::SECTION, 0, self::READ);
			}
		//check monitoring of a specific page
		} elseif ($section == WikiPage::SECTION) {
			$page = WikiPagePeer::retrieveByPk($ref_id);
			if (!$page) {
				$result = false;
			} else {
				$result = $page->userCanRead();
			}
		}
		
		$pr->setResult($result);

		return $pr;
	}

	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(WikiPagePeer::DATABASE_NAME);
		$theme = $params['theme'];

		//get the assigned pages for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();	
		$sql = "
				(SELECT '0' AS wiki_page_id, p.unix_name AS pagename, p.project_id
				FROM monitor m, project p
				WHERE m.user_id = '$user_id' AND m.section='".WikiPlugin::SECTION."'
				AND m.ref_id=p.project_id
				AND project_id NOT IN
					(SELECT p.project_id
					FROM wiki_page wp, monitor m, project p
					WHERE m.user_id = '$user_id' AND m.section='".WikiPage::SECTION."'
					AND m.ref_id=wp.wiki_page_id AND wp.section='project'
					AND wp.ref_id=p.project_id)
				ORDER BY p.project_name ASC)
				UNION
				(SELECT wp.wiki_page_id, wp.pagename, p.project_id
				FROM wiki_page wp, monitor m, project p
				WHERE m.user_id = '$user_id' AND m.section='".WikiPage::SECTION."'
				AND m.ref_id=wp.wiki_page_id AND wp.section='project'
				AND wp.ref_id=p.project_id
				ORDER BY p.project_name ASC)";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$value = '<h3>'.Language::getSessText('Wiki.Monitoring').'</h3>';

		$theader = array();
		$theader[] = 'Project';
		$theader[] = Language::getSessText('My.WikiPages');
		$last_project = null;
		if ($rs->getRecordCount()>0) {
			$theader[] = '';
			$value .= $theme->tableTop($theader);
			$ttype=array();
			while($rs->next()) {
				$ttype[$rs->getInt('project_id')][] = '1';
			}
			$rs->beforeFirst();	
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$project = ProjectPeer::retrieveByPk($rs->getInt('project_id'));
				$project_name = $project->getProjectName();
				$project_unix_name = $project->getUnixName();
				$pagename = $rs->getString('pagename');
				$wiki_page_id = $rs->getString('wiki_page_id');

				if ($last_project != $rs->getInt('project_id')) {
					$row = '<a href="' . GFUrl::getUrl()->projectURL($project_unix_name) . '" >' .  $project_name . '</a>';
					if (WikiPlugin::userIsMonitoring($user, $project)) {
						$row .= '<br />';
						$row .= Language::getSessText('My.MonitoringWiki');
						$row .= ' (<a href="'.GFUrl::getUrl()->getMonitorUrl(WikiPlugin::SECTION, $project->getProjectId(), null, false).'">'.Language::getSessText('My.StopMonitoringWiki').'</a>)';
					}
					$trow[] = $row;
					$tmod[]='rowspan="'.count($ttype[$rs->getInt('project_id')]).'" bgcolor="#FFFFFF"  width="30%"';
				}
				$last_project = $rs->getString('project_id');
				if ($rs->getString('wiki_page_id') <> '0') {		
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL($project_unix_name,'',$this->getName(),'',array('pagename'=>$pagename)) . '" >' .  $pagename . '</a>';
					$tmod[] = '';
					$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(WikiPage::SECTION, $wiki_page_id, null, false).'" >'.Language::getSessText('WikiPage.StopMonitoring').'</a>';					
					$tmod[] = 'align="right"';
				} else {
					$trow[] = '';
					$trow[] = '';
				}
				$value .= $theme->tableRow($trow, $tmod);
			}
		} else {
			$value .= $theme->tableTop($theader);
			$trow = array();
			$trow[] = Language::getSessText('My.NoMonitoredWikiPages');
			$tmod[]='colspan="2"';
			$value .= $theme->tableRow($trow,$tmod);
		}
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}	
	
	private function checkDownloadPerms($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$result = new PluginResult($this);
		$result->setResult(false); // false by default, looking for true
		// if it's for another section, exit
		if ($section!=Wiki::WIKI_FILES_SECTION) {
			$result->setResult(false);
		} else {
			$wp = WikiPagePeer::retrieveByPk($ref_id);
			if (is_object($wp) && $wp->userCanRead()) {
				$result->setResult(true);
			}
		}
		return $result;
	}
	
	public function buildRssFeed($params) {
		if (!isset($params['project']) || !is_object($params['project'])) {
			return;
		}		
		
		if ($params['section'] == 'projectwiki') {
			return $this->buildLatestPagesFeed($params);
		} elseif ($params['section'] == 'projectwikimessages') {
			return $this->buildLatestMessagesFeed($params);
		}
	}
		
	private function buildLatestPagesFeed($params) {
		$project = $params['project'];
		if (!$project->usesPlugin(self::SECTION)) {
			return;
		}
		
		$rss = new RssGenerator();
		$channelName = GFConf::get('system.name').' -'.$project->getProjectName().' - Wiki';
		$url = GFUrl::getUrl();
		
		$channel = array('name' => $channelName,
						'url' => $url->getFullURL($url->baseURL()),
						'description' => 'Gforge Advanced Server RSS');
						
		$items = array();

		// Get the latest modified pages
		$sql = "SELECT wiki_page.* FROM wiki_page, (SELECT wv.wiki_page_id, MAX(wv.last_modified_date) AS max_last_modified_date
				FROM wiki_page AS wp, wiki_version AS wv
				WHERE wv.wiki_page_id=wp.wiki_page_id AND wp.section='project' AND wp.ref_id=".$project->getProjectId()."
				GROUP BY wv.wiki_page_id
				ORDER BY max_last_modified_date DESC
				LIMIT 5 OFFSET 0
			) AS latest_pages
			WHERE latest_pages.wiki_page_id=wiki_page.wiki_page_id
			ORDER BY latest_pages.max_last_modified_date DESC";
		
		$con = Propel::getConnection(WikiVersionPeer::DATABASE_NAME);
		$rs = $con->createStatement()->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		while ($rs->next()) {
			$page = new WikiPage();
			$page->hydrate($rs);
			
			if (!$page->userCanRead()) {
				continue;
			}
			
			$page_link = $url->getFullUrl($url->projectUrl($project, '', 'wiki', '', array('pagename' => $page->getPageName())));
			
			$user_id = $page->getLatestVersion()->getPostedBy();
			$user = UserPeer::retrieveByPk($user_id);
			
			$items[] = array('channel_name' => $channelName,
							'title' => $page->getPageName(),
						    'link' => '<![CDATA['.$page_link.']]>',
						    'author' => $user->getFullName(),
						    'guid' => '<![CDATA['.$page_link.']]>',
						    'description' => '<![CDATA['.$page->getLatestContents().']]>'
				);
		}
		
		$res = new PluginResult($this);
		$res->setResult(array('channel' => $channel, 'items' => $items));
		return $res;
	}
		
	private function buildLatestMessagesFeed($params) {
		$project = $params['project'];
		if (!$project->usesPlugin(self::SECTION)) {
			return;
		}
		
		$rss = new RssGenerator();
		$channelName = $channelName = GFConf::get('system.name').' -'.$project->getProjectName().' - Wiki - Latest messages';
		$url = GFUrl::getUrl();

		$channel = array('name' => $channelName,
						'url' => $url->getFullURL($url->baseURL()),
						'description' => 'Gforge Advanced Server RSS');
		$items = array();
		
		$sql = "SELECT fm.*, wp.* FROM forum_message AS fm, forum_thread AS ft, forum AS f, wiki_page AS wp
				WHERE fm.forum_thread_id=ft.forum_thread_id AND ft.forum_id=f.forum_id AND
					f.section='wikipage' AND f.ref_id=wp.wiki_page_id AND 
					wp.section='project' AND wp.ref_id=".$project->getProjectId()."
				ORDER BY fm.post_date DESC
				LIMIT 5 OFFSET 0";
		$con = Propel::getConnection(WikiVersionPeer::DATABASE_NAME);
		$rs = $con->createStatement()->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		$startcol = (ForumMessagePeer::$numColumns - ForumMessagePeer::$numLazyLoadColumns) + 1;
		while ($rs->next()) {
			$message = new ForumMessage();
			$page = new WikiPage();
			
			$message->hydrate($rs);
			$page->hydrate($rs, $startcol);
			
			if (!$page->userCanRead()) {
				continue;
			}
			$page_link = $url->getFullUrl($url->projectUrl($project, '', 'wiki', '', array('pagename' => $page->getPageName())));

			$user_id = $message->getCreatedBy();
			$user = UserPeer::retrieveByPk($user_id);
			
			$items[] = array('channel_name' => $channelName,
							'title' => $message->getSubject(),
						    'link' => '<![CDATA['.$page_link.']]>',
						    'author' => $user->getFullName(),
						    'guid' => '<![CDATA['.$page_link.']]>',
						    'description' => '<![CDATA['.$message->getBody().']]>',
				);
		}
		$res = new PluginResult($this);
		$res->setResult(array('channel' => $channel, 'items' => $items));
		return $res;
		
	}

	/**
	 * Make the association (if necessary) between the SCM commit and the wiki page
	 */
	private function scmCommit($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$scmcommit = $params['commit'];
		$log = $scmcommit->getMessageLog();
		
		// $params['associations'] has all the associations
		
		foreach ($params['associations'] as $association) {
			list($section, $ref_id) = $association;
			if ($section != WikiPage::SECTION) {
				continue;
			}
			
			$wiki_page_id = $ref_id;
	
			// see if the item belongs to the project
			$wiki_page = WikiPagePeer::retrieveByPk($wiki_page_id);
			if (is_null($wiki_page)) continue;
			
			if ($wiki_page->getSection() != Project::SECTION || $wiki_page->getRefId() != $project->getProjectId()) {
				// The referenced page is not part of this project
				continue;
			}
			
			// Message belongs to the project. Add entry to the association table.
			try {
				$association = new Association();
				$association->setSection(ScmCommit::SECTION);
				$association->setRefId($scmcommit->getPrimaryKey());
				$association->setToSection(WikiPage::SECTION);
				$association->setToRefId($wiki_page_id);
				$association->setComment($scmcommit->getMessageLog());
				$association->save();
			} catch (Exception $ex) {
				// This means the association is duplicated. Ignore the exception.
			}
		}
		
		return null;
	}
	function getProjectSettingsForm($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) {
			return;
		}
		
		$setting = WikiProjectSettingPeer::retrieveByPk($project->getProjectId());
		if (!$setting) {
			$show_associations_tab = true;
			$show_tags_tab = true;
			$wiki_type = WikiProjectSetting::HTML_WIKI;
		} else {
			$show_associations_tab = $setting->getShowAssociationsTab();
			$show_tags_tab = $setting->getShowTagsTab();
			$wiki_type = $setting->getWikiType();
		}
		
		$form = array();
		$form['title'] = Language::getSessText('Wiki.ProjectSettingsTitle');
		$form['contents'] = '
			<input type="checkbox" disabled="disabled" name="wiki_show_associations_tab" '.($show_associations_tab ? 'checked="checked"' : '').'/> '.Language::getSessText('Tabs.ShowAssociationsTab').' <br /> 
			<input type="checkbox" name="wiki_show_tags_tab" '.($show_tags_tab ? 'checked="checked"' : '').'/> '.Language::getSessText('Tabs.ShowTagsTab').' <br />
			<strong>'.Language::getSessText('Wiki.Admin.WikiTypeTitle').':</strong><br />
			<input type="radio" name="wiki_type" value="'.WikiProjectSetting::HTML_WIKI.'" '.($wiki_type == WikiProjectSetting::HTML_WIKI ? 'checked="checked"' : '').' /> '.Language::getSessText('Wiki.Admin.UseHtmlWiki').' <br />
			<input type="radio" name="wiki_type" value="'.WikiProjectSetting::TEXT_WIKI.'" '.($wiki_type == WikiProjectSetting::TEXT_WIKI ? 'checked="checked"' : '').' /> '.Language::getSessText('Wiki.Admin.UseTextWiki').' <br />
			<em>'.Language::getSessText('Wiki.Admin.WikiTypeTip').'</em>
			<br />
		';
		
		$result = new PluginResult($this);
		$result->setResult($form);
		return $result;
	}
	
	function saveProjectSettings($params) {
		$project = $params['project'];
		$setting = WikiProjectSettingPeer::retrieveByPk($project->getProjectId());
		if (!$setting) {
			$setting = new WikiProjectSetting();
			$setting->setProject($project);
		}
		
		$setting->setShowAssociationsTab( (isset($_POST['wiki_show_associations_tab']) ? true : false) );
		$setting->setShowTagsTab( (isset($_POST['wiki_show_tags_tab']) ? true : false) );
		$setting->setWikiType(Validator::sanitizeInt($_POST['wiki_type']));
		$setting->save();
	}
}

?>
