<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * MS Project plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once 'session/LoginFactory.php';

function continueSession($sessionHash) {
	$login = LoginFactory::getLogin();
	$ip = $_SERVER['REMOTE_ADDR'];
	$session = Session::getSession();
	$session->setup($sessionHash, $ip);
	return $session;
}


/**
return data:

	$array[success]=true;
	$array[session_hash]=jkjkjkjkjkjkjkj;
	$array[subprojects]=array(
						55=>'Subproject 1', 
						77=>'Subproject2'
						);

**OR**

	$array[success]=false;
	$array[errormessage]='Bad Password';
 */
function MSPLogin($username, $password) {
	$login = LoginFactory::getLogin();
	$ip = $_SERVER['REMOTE_ADDR'];
	try {
		$userSession = $login->doLogin($username, $password);
		$session = Session::getSession();
		$session->setUserSession($userSession, $ip);
		$sessionHash = $session->getSessionHash();
		Session::getSession()->setup($sessionHash, $ip);

		$array['success'] = true;
		$array['session_hash'] = $sessionHash;
		$array['subprojects'] = array();

		// Get All User's Projects
		$user = $session->getUser();
		
		$projects = $user->getAllProjects();
		
		// Check if we have Projects in the variable
		if (count($projects) > 0) {
			// Iterate the $projects variable
			foreach($projects as $project) {
				// Now get the Project's Trackers			
				$project_name = $project->getName();
				$project_id = $project->getId();

				$c = new Criteria();
				$c->add(TrackerPeer::PROJECT_ID, $project_id);
				$c->add(TrackerPeer::DATATYPE, Tracker::DATATYPE_TASK);
				$trackers = TrackerPeer::doSelect($c);
				// Check if we have Trackers in the variable
				// Iterate the $trackers variable
				foreach($trackers as $tracker) {
					$item = new TrackerItem();
					$tracker->addTrackerItem($item);
					if ($item->userCanCreate()) {
						$tracker_id = $tracker->getTrackerId();
						$tracker_name = $project_name .': '. $tracker->getTrackerName();
						$array['subprojects'][$tracker_id]=$tracker_name;
					}
					unset($item);
				}
			}
			if (count($array['subprojects']) == 0) {
				$array['success'] = false;
				$array['errormessage'] = 'No Subprojects Found (Trackers)';
			}
		} else {
			$array['success'] = false;
			$array['errormessage'] = 'No Subprojects Found (Projects)';
		}

	} catch (InvalidDataException $ex) {
		$array['success'] = false;
		$errors = $ex->getErrors();
		$error = implode("\n", array_values($errors));
		$array['errormessage'] = $error;
	} catch (Exception $e) {
		$error = $e->getMessage();
		$array['success'] = false;
		$array['errormessage'] = $error;
	}

	//print_r($array, 'MSPLogin::return-array');
	return $array;
}


/**
* MSPGetProjects
* Return the projects by user.
*
* @author	Luis Hurtado	luis@gforgegroup.com
* @param	session_hash	User session
* @return	Groups		User groups
* @date		2005-01-19
*
*/
function MSPGetProjects($session_hash) {
	try {
		$session = continueSession($session_hash);
		// Get All User's Projects
		$user = $session->getUser();
		$c = new Criteria();
		$c->add(UserProjectPeer::USER_ID, $user->getUserId());
		$projects = UserProjectPeer::doSelect($c);
		// Check if we have Projects in the variable
		if (count($projects) > 0) {
			// Iterate the $projects variable
			foreach($projects as $userProject) {
				$project = $userProject->getProject();
				$project_name = $project->getName();
				$project_id = $project->getId();
				$array['projects'][$project_id]=$project_name;
			}
			$array['success'] = true;
		} else {
			$array['success'] = false;
			$array['errormessage'] = 'No Projects Found';
		}
	} catch (InvalidDataException $ex) {
		$array['success'] = false;
		$errors = $ex->getErrors();
		$error = implode("\n", array_values($errors));
		$array['errormessage'] = $error;
	} catch (Exception $ex) {
		$array['success'] = false;
		$error = $ex->getMessage();
		$array['errormessage'] = $error;
	}
	return $array;
}


/**
* MSPCreateProject
* Create SubProjects
*
* @author	Luis Hurtado	luis@gforgegroup.com
* @param	groupid		ID Group
* @param	session_hash	User Session
* @param	name		Project name
* @param	ispublic	1 Public  0 Private
* @param	description	Project Description
* @return	ProjectGroup	Object ProjectGroup
* @date		2005-01-19
*
*/
function MSPCreateProject($groupid, $session_hash, $name, $ispublic, $description) {
	try {
		$session = continueSession($session_hash);
		$Project = ProjectPeer::retrieveByPk($groupid);
		if (!is_object($Project) || !$Project->getPermission()->isAdmin()) {
			$res['code'] = 'error';
			$res['description'] = 'Permission Denied: Cannot Create Project Groups';
			return $res;
		}

		$Tracker = new Tracker();
		$Tracker->setProject($Project);
		$Tracker->setTrackerName(Validator::sanitizeString($name));
		$Tracker->setDescription(Validator::sanitizeString($description));
		$Tracker->setIsPublic(($ispublic == 0) ? false : true );
		$Tracker->setRestrictBrowse(false);
		$Tracker->setEmailAllUpdates(false);
		$Tracker->setEmailAddress('');
		$Tracker->setSubmitInstructions('');
		$Tracker->setBrowseInstructions('');
		$Tracker->setDuePeriod(15*60*60*24);
		$Tracker->setDatatype(Tracker::DATATYPE_TASK);
		$Tracker->save();
		$Tracker->buildDefaultFields();
//		ActionManager::notify('TRACKER_ADD_ACTION', $Tracker);
		return $Tracker;
	} catch (InvalidDataException $ex) {
		$res['code'] = 'error';
		$errors = $ex->getErrors();
		$error = implode("\n", array_values($errors));
		$res['description'] = $error;
		return $res;
	} catch (Exception $ex) {
		$res['code'] = 'error';
		$error = $ex->getMessage();
		$res['description'] = $error;
		return $res;
	}
}

/**

return data:

	$array[success]=true;
	$array[tasks]=array of ProjectTask objects

**OR**

	$array[success]=false;
	$array[errormessage]='Invalid Subproject';

*/
function MSPDownload($session_hash, $group_project_id) {
	try {
		$session = continueSession($session_hash);

		$Tracker = TrackerPeer::retrieveByPk($group_project_id);
		if (!$Tracker || !is_object($Tracker)) {
			$array['success'] = false;
			$array['errormessage'] = 'Could Not Get ProjectGroup';
			return $array;
		}

		$c = new Criteria();
		$c->add(TrackerItemPeer::TRACKER_ID, $Tracker->getTrackerId());
		$c->addAscendingOrderByColumn(TrackerItemPeer::SORT_ORDER);
		$TrackerItems = TrackerItemPeer::doSelect($c);

		if (count($TrackerItems) == 0) {
			$array['success'] = false;
			$array['errormessage'] = 'No Matching ProjectTasks Found';
		}
		else {
			$array['success'] = true;
			$array['tasks'] = $TrackerItems;
		}
		return $array;
	} catch (InvalidDataException $ex) {
		$array['success'] = false;
		$errors = $ex->getErrors();
		$error = implode("\n", array_values($errors));
		$array['errormessage'] = $error;
		return $array;
	} catch (Exception $ex) {
		$array['success'] = false;
		$error = $ex->getMessage();
		$array['errormessage'] = $error;
		return $array;
	}
}


//same as download
//function MSPGetLatest($session_hash,$group_project_id) {
//}

/**

LINK TYPES -
				SS . Start to Start
				SF . Start to Finish  
				FS . Finish to Start
				FF . Finish to Finish

	params:

	$session_hash
	$group_project_id (subproject_id)
	$tasks = 
		array(
			id=>1,
			msproj_id=>p1,
			parent_id=>4,
			parent_msproj_id=>p5
			name=>'Task Name',
			duration=>5,
			work=>40,
			start_date=>'10/1/04',
			end_date=>'10/8/04',
			percent_complete=>'50',
			priority=>'medium',
			resources=>array(
				array(user_name=>'unix_name'),
				...
			),
			dependenton=>array(
				array( 'task_id'=>'55', 'msproj_id'=44, 'task_name'=>'Task Name', 'link_type'='SS' ),
				...
			)
			notes=>'notes'
		),

Return:

	$array[success]=true;

**OR**

	$array[success]=false;
	$array[errormessage]='Invalid Subproject';
	$array[resourcename]=array(
			'Michael',
			'Jon',
			...
		)
	$array[usernames]=array(
			array(user_id=>55,user_name='Jon Doe'),
			array(user_id=>87,user_name='Foo'),
			...
		)

*/
function MSPCheckin($session_hash, $group_project_id, $tasks) {
	try {
		$session = continueSession($session_hash);
	
		$Tracker = TrackerPeer::retrieveByPk($group_project_id);
		if (!$Tracker || !is_object($Tracker)) {
			$array['success'] = false;
			$array['errormessage'] = 'Could Not Get ProjectGroup';
			return $array;
		} else {
			return importTasks($session, $Tracker, $tasks);
		}
	} catch (InvalidDataException $ex) {
		$array['success'] = false;
		$errors = $ex->getErrors();
		$error = implode("\n", array_values($errors));
		$array['errormessage'] = $error;
		return $array;
	} catch (Exception $ex) {
		$array['success'] = false;
		$error = $ex->getMessage();
		$array['errormessage'] = $error;
		return $array;
	}
}


function importTasks($session, $Tracker, $tasks) {
	global $plugin_version;
	
	$was_error = false;
	$techs = $Tracker->getTechnicians();
	$count = count($tasks);
	
	$tarr = array();
	foreach ($techs as $tech) {
		$tarr[strtolower($tech->getUnixName())] = $tech->getUserId();
		$tarr[strtolower($tech->getFullName())] = $tech->getUserId();
	}

	//  Create a linked list based on the msproj_id
	$invalid_names = array();
	$msprojid = array();
	$count=count($tasks);
	for ($i=0;$i<$count;$i++) {
		$msprojid[$tasks[$i]['msproj_id']] = & $tasks[$i];
		$resources = $tasks[$i]['resources'];
		if (isset($resources) && is_array($resources)) {
			foreach ($resources as $resource) {
				//validate user - see if they really exist as techs in this subproject
				if (!isset($tarr[strtolower($resource['user_name'])])) {
					//create list of bogus names to send back
					if (array_search($resource['user_name'], $invalid_names) === false) {
						$invalid_names[] = $resource['user_name'];
					}
				}
			}
		}
	}
	//  invalid assignees - send it back for remapping
	if (count($invalid_names)) {
		$array['success'] = false;
		$array['errormessage'] = 'Invalid Resource Name';
		$array['resourcename'] = $invalid_names;
		$array['usernames'] = array();
		foreach ($techs as $tech) {
			$array['usernames'][$tech->getUserId()] = $tech->getUnixName();
		}
		return $array;
	} else {
		//  Begin inserting/updating the tasks
		for ($i=0;$i<$count;$i++) {
			if ($was_error) { continue; }
			$is_new = false;

			// If task id is not set, or is lower than zero we create a new task object
			if (!isset($tasks[$i]['id']) || $tasks[$i]['id'] < 1) {
				$is_new = true;
				$TrackerItem = new TrackerItem();
			} else {
			// if task id is present and apparently valid, we try to get the task object
				$TrackerItem = TrackerItemPeer::retrieveByPk(($tasks[$i]['id']));
				//if the task is not a valid object, we create a new task object
				if (!$TrackerItem || !is_object($TrackerItem)) {
					$is_new = true;
					$TrackerItem = new TrackerItem();
				}
			}

			if ($is_new) {
				if (!isset($tasks[$i]['notes']) || strlen($tasks[$i]['notes']) < 5) {
					if (version_compare($plugin_version, '1.3', '=') === true) {
						$tasks[$i]['notes'] = base64_encode('None Provided');
					} else if (version_compare($plugin_version, '1.3', '<') === true) {
						$tasks[$i]['notes'] = 'None Provided';
					}
				}

				$Tracker->addTrackerItem($TrackerItem);
				if (!$TrackerItem->userCanCreate()) {
					$was_error = true;
					$array['success'] = false;
					$array['errormessage'] = 'Permission Denied: Cannot Create Tasks';
				}
			
				//remap priority names=>numbers
				$priority = $tasks[$i]['priority'];
				if (!$priority || $priority < 1 || $priority > 5) {
					$priority = 3;
				}
				//map users
				$assignees = array();
				$resources = $tasks[$i]['resources'];
				if (is_array($resources)) {
					foreach ($resources as $resource) {
						//get their user_id from the $tarr we created earlier
						$uid = $tarr[strtolower($resource['user_name'])];
						$eff = $resource['percent_effort'];
						$assignees[$uid] = $eff;
					}
				}
				if (count($assignees) == 0) {
					$assignees[100] = 0;
				}

				$hours = (isset($tasks[$i]['work'])) ? $tasks[$i]['work'] : '0.0';
				$percent_complete = (isset($tasks[$i]['percent_complete'])) ? $tasks[$i]['percent_complete'] : 0;
				if ($percent_complete == '') {
					$percent_complete = 0;
				} else if ($percent_complete > 100) {
					$percent_complete = 100;
				}
				try {
					//$hours
					$TrackerItem->setPriority(Validator::sanitizeInt($priority));
					$TrackerItem->setSubmittedBy($session->getUser()->getUserId());
					if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
						$TrackerItem->setOpenDate(Validator::sanitizeDate($tasks[$i]['start_date']));
						$TrackerItem->setCloseDate(Validator::sanitizeDate($tasks[$i]['end_date']));
					}
					$TrackerItem->setSummary(decodeNotes($tasks[$i]['name']));
					$TrackerItem->setDetails(decodeNotes($tasks[$i]['notes']));
					$TrackerItem->addAssignees($assignees);
					//$TrackerItem->addRelations($deps);

					$extra_ids = $TrackerItem->getTracker()->getTrackerExtraFields();
					$duration_id = 0;
					$complete_id = 0;
					$effort_id   = 0;
					$extraFields = array();

					$criteria = new Criteria();
					$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
					$defaultValues = $TrackerItem->getTracker()->getTrackerDefaultSelected($criteria);

					foreach ($extra_ids as $field) {
						$efid = $field->getTrackerExtraFieldId();
						if ($field->getAlias() == 'duration') { $duration_id = $efid; }
						else if ($field->getAlias() == 'percentcomplete') { $complete_id = $efid; }
						else if ($field->getAlias() == 'estimatedeffort') { $effort_id = $efid; }
						else {
							if ($field->getFieldType()==TrackerExtraField::TYPE_MULTISELECT 
								|| $field->getFieldType()==TrackerExtraField::TYPE_CHECKBOX) {
								$extraFields[$efid] = isset($defaultValues[$efid]) ? $defaultValues[$efid] : array(100);
							} else if ($field->getFieldType()==TrackerExtraField::TYPE_SELECT 
								|| $field->getFieldType()==TrackerExtraField::TYPE_RADIO
								|| $field->getFieldType()==TrackerExtraField::TYPE_STATUS
								|| $field->getFieldType()==TrackerExtraField::TYPE_RELEASE) {
								$extraFields[$efid] = isset($defaultValues[$efid]) ? $defaultValues[$efid][0] : 100;
							} else {
								$extraFields[$efid] = '';
							}
						}
					}

					if ($duration_id > 0) {
						$extraFields[$duration_id] = ($tasks[$i]['duration'] / 8);
					}
					if ($complete_id > 0) {
						$extraFields[$complete_id] = $percent_complete;
					}
					if ($effort_id > 0) {
						$extraFields[$effort_id] = $tasks[$i]['work'];
					}

					if (count($extraFields) > 0) {
						$TrackerItem->addExtraFields($extraFields);
					}

					$TrackerItem->setSortOrder(Validator::sanitizeInt($tasks[$i]['msproj_id']));
					
					// TODO: $tasks[$i]['parent_id']
					// $deps = array()
					$TrackerItem->save();
					ActionManager::notify('TRACKER_ITEM_ADD_ACTION',$TrackerItem);

					$tasks[$i]['id']  = $TrackerItem->getTrackerItemId();
					$tasks[$i]['task_id']  = $TrackerItem->getTrackerItemId();
					$tasks[$i]['obj'] = $TrackerItem;
					///$TrackerItem = null;  // TODO: DO WE HAVE TO DO THIS?
				} catch (InvalidDataException $ex) {
					$array['success'] = false;
					$was_error = true;
					$errors = $ex->getErrors();
					$error = implode("\n", array_values($errors));
					$array['errormessage'] = $error;
				} catch (Exception $ex) {
					$array['success'] = false;
					$was_error = true;
					$error = $ex->getMessage();
					$array['errormessage'] = 'Error Creating ProjectTask: '.$error;
					break 1;
				}
			} else {
				//update existing task
				//create the task
				if (!$TrackerItem->userCanUpdate() ||
					$TrackerItem->getTracker()->getTrackerId() != $Tracker->getTrackerId()) {
					$array['success'] = false;
					$was_error = true;
					$array['errormessage'] = 'Permission Denied: Cannot Update Tasks or Project Group Is Invalid';
				} else {
					// Add notes as messages
					if (isset($tasks[$i]['notes'])) {
						$TrackerItem = addMessage($TrackerItem, decodeNotes($tasks[$i]['notes']), $session);
					}

					//remap priority names=>numbers
					$priority = $tasks[$i]['priority'];
					if (!$priority || $priority < 1 || $priority > 5) {
						$priority = 3;
					}
					//map users
					$assignees = array();
					$resources = (is_array($tasks[$i]['resources'])) ? $tasks[$i]['resources'] : array();

					foreach ($resources as $resource) {
						//get their user_id from the $tarr we created earlier
						$uid = $tarr[strtolower($resource['user_name'])];
						$eff = $resource['percent_effort'];
						$assignees[$uid] = $eff;
					}					
					if (count($assignees) == 0) {
						$assignees[100] = 0;
					}

					$hours = (isset($tasks[$i]['work'])) ? $tasks[$i]['work'] : '0.0';
					$percent_complete = (isset($tasks[$i]['percent_complete'])) ? $tasks[$i]['percent_complete'] : 0;
					if ($percent_complete == '') {
						$percent_complete = 0;
					} elseif ($percent_complete > 100) {
						$percent_complete = 100;
					}

					try {
						//$hours
						$TrackerItem->setPriority(Validator::sanitizeInt($priority));
						if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
							$TrackerItem->setOpenDate(Validator::sanitizeDate($tasks[$i]['start_date']));
							$TrackerItem->setCloseDate(Validator::sanitizeDate($tasks[$i]['end_date']));
						}
						$TrackerItem->setSummary(decodeNotes($tasks[$i]['name']));
						$TrackerItem->updateAssignees($assignees);

						//$TrackerItem->updateRelations($deps);
						
						$extra_ids = $TrackerItem->getTracker()->getTrackerExtraFields();
						$duration_id = 0;
						$complete_id = 0;
						$effort_id   = 0;
						$extraFields = $TrackerItem->getArrangedExtraFieldData();

						foreach ($extra_ids as $field) {
							if ($field->getAlias() == 'duration') { $duration_id = $field->getTrackerExtraFieldId(); }
							if ($field->getAlias() == 'percentcomplete') { $complete_id = $field->getTrackerExtraFieldId(); }
							if ($field->getAlias() == 'estimatedeffort') { $effort_id = $field->getTrackerExtraFieldId(); }
						}
						if ($duration_id > 0) {
							$extraFields[$duration_id] = ($tasks[$i]['duration'] / 8);
						}
						if ($complete_id > 0) {
							$extraFields[$complete_id] = $percent_complete;
						}
						if ($effort_id > 0) {
							$extraFields[$effort_id] = $tasks[$i]['work'];
						}

						if (count($extraFields) > 0) {
							$TrackerItem->updateExtraFields($extraFields);
						}
			
						// TODO: $tasks[$i]['parent_id']
						// $deps = array()

						$TrackerItem->setLastModifiedDate(time());
						$TrackerItem->setLastModifiedBy($session->getUser()->getUserId());
						$TrackerItem->setSortOrder(Validator::sanitizeInt($tasks[$i]['msproj_id']));

						$TrackerItem->save();
						ActionManager::notify('TRACKER_ITEM_EDIT_ACTION',$TrackerItem);

						$tasks[$i]['id']  = $TrackerItem->getTrackerItemId();
						$tasks[$i]['task_id']  = $TrackerItem->getTrackerItemId();
						$tasks[$i]['obj'] = $TrackerItem;
						///$TrackerItem = null;  // TODO: DO WE HAVE TO DO THIS?
					} catch (InvalidDataException $ex) {
						$array['success'] = false;
						$was_error = true;
						$errors = $ex->getErrors();
						$error = implode("\n", array_values($errors));
						$array['errormessage'] = 'Error Updating ProjectTask: '.$error;
						break 1;
					} catch (Exception $ex) {
						$array['success'] = false;
						$was_error = true;
						$error = $ex->getMessage();
						$array['errormessage'] = 'Error Updating ProjectTask: '.$error;
						break 1;
					}
				}
			} //if task_id
			//accumulate list of completed tasks
			//any task not in this list will be deleted
			$completed[$tasks[$i]['id']] = true;
		} // for

		if (!$was_error) {
			//iterate the tasks
			for ($i=0; $i<$count; $i++) {
				//dependencies
				$darr=(isset($tasks[$i]['dependenton'])) ? $tasks[$i]['dependenton'] : array();
				$deps=array();
				//iterate each dependency in a task
				for ($dcount=0; $dcount<count($darr); $dcount++) {
					//get the id of the task we're dependent on -
					// may have to get it from msprojid linked list
					$id=(isset($darr[$dcount]['task_id']) && $darr[$dcount]['task_id'] > 0) 
						? $darr[$dcount]['task_id'] : $msprojid[$darr[$dcount]['msproj_id']]['id'];
				
					//prevent task from being dependent on itself
					if ($id == $tasks[$i]['id']) {
						continue;
					}
					$deps[$id]=$darr[$dcount]['link_type'];
				}
				
				//parent_task_id
				if ($tasks[$i]['parent_msproj_id'] > 0) {
					$id = $msprojid[$tasks[$i]['parent_msproj_id']]['id'];
					$tasks[$i]['obj']->setParentId($id);
				}
				
				if (is_object($tasks[$i]['obj'])) {
					try {
						$tasks[$i]['obj']->updateRelations($deps);
						$tasks[$i]['obj']->save();
					} catch (Exception $ex) {
						$was_error=true;
						$array['success']=false;
						$array['errormessage']='Error updating relations: '. $ex->getMessage();
					}
				} else {
					//printr($foo,'PROJECT TASK OBJECT DOES NOT EXIST IN OBJ ARRAY');
				}
				unset($deps);
			} //iterates tasks to do dependencies
		}
		
		//	Delete unreferenced tasks
		if (!$was_error) {
			$c = new Criteria();
			$c->add(TrackerItemPeer::TRACKER_ID, $Tracker->getTrackerId());
			$trackerItems = TrackerItemPeer::doSelect($c);

			foreach ($trackerItems as $myItem) {
				$id = $myItem->getTrackerItemId();
				if (!isset($completed[$id]) && $myItem->userCanDelete()) {
					try {
						$myItem->delete();
					} catch (Exception $ex) {
						//
					}
				}
			}
		}
	} //invalid names

	if (!$was_error) {
		$array['success'] = true;
	}
	return $array;
}

function getPriority($p) {
	$p = strtolower($p);
	$ret = 3;
	switch ($p) {
		case 'very low' : 
			$ret = Tracker::PRIORITY_VERYLOW;
			break;
		case 'low' : 
			$ret = Tracker::PRIORITY_LOW;
			break;
		case 'medium' : 
			$ret = Tracker::PRIORITY_MEDIUM;
			break;
		case 'high' : 
			$ret = Tracker::PRIORITY_HIGH;
			break;
		case 'very high' : 
			$ret = Tracker::PRIORITY_VERYHIGH;
			break;
	}
	return $ret;	
}

function decodeNotes($notes) {
	global $plugin_version;

	if (version_compare($plugin_version, '1.3', '>=') === true) {
		$notes = base64_decode($notes);
	}
	$notes = mb_convert_encoding($notes, 'UTF-8');
	$notes = html_entity_decode($notes);
	return Validator::sanitizeString($notes);
}

function addMessage($TrackerItem, $notes, $session) {
	$body = trim($notes);
	if (strlen($body) < 5) {
		// we don't do anything
		return $TrackerItem;
	} else if ($TrackerItem->getDetails() == $body || $body == 'None Provided') {
		// we don't do anything
		return $TrackerItem;
	} else {
		$is_new = true;
		$messages = $TrackerItem->getTrackerItemMessages();
		foreach ($messages as $message) {
			if ($body == trim($message->getBody()) &&
				$session->getUser()->getUserId() == $message->getSubmittedBy()) {
				$is_new = false;
			}
		}
		if ($is_new) {
			$tim = new TrackerItemMessage();
			$TrackerItem->addTrackerItemMessage($tim);
			$tim->setBody($body);
			$tim->setSubmittedBy($session->getUser()->getUserId());
			$tim->save();
		}
		return $TrackerItem;
	}
}
?>
