<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerItemScmCommit

require_once('plugins/tracker/lib/TrackerItemScmCommit.php');
require_once('plugins/tracker/lib/TrackerItemScmCommitPeer.php');
require_once('plugins/scm/lib/ScmCommit.php');
require_once('plugins/scm/lib/ScmCommitPeer.php');

require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerItemScmCommit',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int'),
		'scm_commit_id' => array('name'=>'scm_commit_id', 'type' => 'xsd:int'),
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'commit_date' => array('name'=>'commit_date', 'type' => 'xsd:string'),
		'message_log' => array('name'=>'message_log', 'type' => 'xsd:string'),
		'user_id' => array('name'=>'user_id', 'type' => 'xsd:int')
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerItemScmCommit',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerItemScmCommit[]')),
	'tns:TrackerItemScmCommit'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerItemScmCommits',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'scm_commit_id' => 'xsd:int',
	), 
	array('getTrackerItemScmCommits' => 'tns:ArrayOfTrackerItemScmCommit'),
	$uri, 
	$uri.'#getTrackerItemScmCommits',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerItemScmCommit
 * @param	$tracker_item_id	Filter by tracker_item_id
 * @param	$scm_commit_id	Filter by scm_commit_id
 */
function getTrackerItemScmCommits($sessionHash, $tracker_item_id, $scm_commit_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerItemScmCommits','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c = new Criteria();
	$c->add(TrackerItemScmCommitPeer::TRACKER_ITEM_ID, $tracker_item_id);
	if ($scm_commit_id >= 0) {
		$c->add(TrackerItemScmCommitPeer::SCM_COMMIT_ID, $scm_commit_id);
	}

	$objs = TrackerItemScmCommitPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerItemScmCommitToSOAP($obj);
	}
	return $res;
}

function TrackerItemScmCommitToSOAP(&$obj) {
	$ScmObj = ScmCommitPeer::retrieveByPK($obj->getScmCommitId());
	$ret = array();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['scm_commit_id'] = $obj->getScmCommitId();
	$ret['project_id'] = $ScmObj->getProjectId();
	$ret['commit_date'] = $ScmObj->getCommitDate();
	$ret['message_log'] = $ScmObj->getMessageLog();
	$ret['user_id'] = $ScmObj->getUserId();
	return $ret;
}
?>